@@ -1,4 +1,3 @@
-
 // -*- mode: c++; c-basic-offset:4 -*-
 
 // This file is part of libdap, A C++ implementation of the OPeNDAP Data
@@ -37,7 +36,7 @@
 
 #include "config_nc.h"
 
-static char rcsid[] not_used ={"$Id: Dattr.cc 12258 2005-09-27 19:38:28Z jimg $"};
+static char rcsid[] not_used ={"$Id: Dattr.cc 15125 2006-11-21 23:12:00Z jimg $"};
 
 #include "Dnetcdf.h"
 #include "Dncx.h"
@@ -387,26 +386,12 @@ nc_inq_att(int cdfid, int varid, const char *name,
 	  case Attr_float64:
 	    if(datatypep) *datatypep =  NC_DOUBLE;
 	    break;
-	  case Attr_string: {
+	  case Attr_string:
+	  case Attr_url:
 	    if(datatypep) *datatypep =  NC_CHAR;
-	    if(lenp) {
-		*lenp = 0;
-		// string lenght is the total number of bytes plus
-		// terminating character. There is a '\n' at the end of each
-		// string, but a '\0' at the end of the attibutes.
-		unsigned int attr_siz = attr.get_attr_num(p);
-		
-		for (unsigned int i = 0; i < attr_siz; i++){
-		    attrV = attr.get_attr(p, i);
-		    *lenp +=  attrV.length() - 1; // EOS included, quotes removed  
-		}
-		// single string (coming from a netCDF server) remove EOS
-		// for compatibility
-		if (attr.get_attr_num(p) == 1)
-		    *lenp -= 1;
-	    }
+	    if(lenp)
+		*lenp = compute_string_attr_length(attr, p);
 	    break;
-	  }
 
 	  default:
 	    return NC_ENOTATT;
@@ -451,21 +436,9 @@ nc_inq_attlen(int cdfid, int varid, const char *name, size_t *lenp)
 	if (p == attr.attr_end())
 	    return NC_ENOTATT;
 
-	if (attr.get_attr_type(p) == Attr_string) {
-	    *lenp = 0;
-	    // string lenght is the total number of bytes plus
-	    // terminating character. There is a '\n' at the end of each
-	    // string, but a '\0' at the end of the attibutes.
-	    unsigned int attr_siz = attr.get_attr_num(p);
-	    string attrV;
-	    for (unsigned int i = 0; i < attr_siz; i++){
-		attrV = attr.get_attr(p, i);
-		*lenp +=  attrV.length() - 1; // EOS included, quotes removed  
-	    }
-	    // single string (coming from a netCDF server) remove EOS
-	    // for compatibility
-	    if (attr.get_attr_num(p) == 1)
-		*lenp -= 1;
+	if (attr.get_attr_type(p) == Attr_string
+	    || attr.get_attr_type(p) == Attr_url) {
+	    *lenp = compute_string_attr_length(attr,p);
 	}
 	else {
 	    *lenp = static_cast<size_t>(attr.get_attr_num(p));
@@ -530,7 +503,9 @@ nc_inq_atttype(int cdfid, int varid, const char *name, nc_type *datatypep)
 	    *datatypep =  NC_DOUBLE;
 	    break;
 	  case Attr_string:
+      case Attr_url:
 	    *datatypep =  NC_CHAR;
+        break;
 	  default:
 	    return NC_NOERR;	// Fails three tests w/o this. 03/09/04 jhrg
 	}
@@ -590,6 +565,7 @@ nc_copy_att(int incdf, int invar, const char *name, int outcdf, int outvar)
     rcode = nc_put_att((*conns)[outcdf]->get_ncid(), outvar,  
 		       name, datatype, count, (void *)values);
 #endif
+    DBG(cerr << "nc_copy_att" << endl);
     rcode = nc_put_att(outcdf, outvar, name, datatype, count, (void *)values);
 
     delete [] values; values = 0;
@@ -624,8 +600,16 @@ nc_get_att_text(int cdfid, int varid, const char *name, char *str)
 	if (dt != NC_CHAR)
             throw Error(NC_ECHAR, "NetCDF Error");
 	    // return NC_ECHAR;
-	rcode = putn_into_text((void **)&values, count, str);
 
+	DBG(cerr << "nc_get_att_text" << endl);
+#if 0
+	memcpy(str, values, count);
+	str[count + 1] = '\0';	// ?? Is there space for this. seems to be
+				// needed. jhrg 11/10/06
+	rcode = NC_NOERR;
+#else
+	rcode = putn_into_text((void **)&values, count, str);
+#endif
 	delete [] values; values = 0;
 	return rcode;
     }
@@ -658,7 +642,7 @@ nc_get_att_schar(int cdfid, int varid, const char *name, signed char *tp)
 	if (dt == NC_CHAR)
             throw Error(NC_ECHAR, "NetCDF Error");
 	    // return NC_ECHAR;
-
+	DBG(cerr << "nc_get_att_schar" << endl);
 	rcode = putn_into_schar((void **)&values, count, tp, dt);
 
 	delete [] values; values = 0;
@@ -894,7 +878,10 @@ nc_get_att(int cdfid, int varid, const char *name, void *values)
     try {
         size_t count;
         nc_type datatype;
-        values = (*conns)[cdfid]->get_raw_values(varid, name, &count, &datatype);
+        char *v = (*conns)[cdfid]->get_raw_values(varid, name, &count, &datatype);
+	DBG(cerr << "nc_get_att" << endl);
+	memcpy(values, v, count * nctypelen(datatype));
+	delete[] v; v = 0;
     }
     catch (Error &e) {
         set_opendap_error(e);
