@@ -45,7 +45,7 @@
 
 #include "config_nc.h"
 
-static char rcsid[] not_used ={"$Id: Dnc.cc 11906 2005-08-08 19:51:43Z root $"};
+static char rcsid[] not_used ={"$Id: Dnc.cc 14988 2006-11-01 22:21:09Z jimg $"};
 
 #include <string>
 
@@ -92,8 +92,12 @@ int
 nc__create_mp(const char * path, int ioflags, size_t initialsz, int basepe,
 	size_t *chunksizehintp, int *ncid_ptr)
 {
-    ClientParams cp(path);
-    NCConnect *c = new NCConnect(cp.url_without_params(), cp);
+    // ClientParams is allocated outside of NCConnect so that NCConnect's
+    // ctor can get the path stripped of the URL params. It then passes this
+    // URL to AISConnect. The pointer 'cp' is passed to NCConnect so that the
+    // memory can be recovered when this instance is deleted.
+    ClientParams *cp = new ClientParams(path);
+    NCConnect *c = new NCConnect(cp->url_without_params(), cp);
 
 #ifdef LOCAL
     if(c->is_local()) {  
@@ -114,13 +118,13 @@ nc__create_mp(const char * path, int ioflags, size_t initialsz, int basepe,
     	    return NC_NOERR;
     	}
     	else {
-            delete c;
+            delete c; c = 0;
     	    return rcode;
     	}
     }
 #endif
 
-    delete c;
+    delete c; c = 0;
     return NC_EPERM;
 }
 
@@ -139,6 +143,16 @@ nc_create(const char * path, int ioflags, int *ncid_ptr)
         return nc__create(path, ioflags, 0, NULL, ncid_ptr);
 }
 
+/* This function sets a default create flag that will be logically
+   or'd to whatever flags are passed into lnc_create for all future
+   calls to lnc_create.
+   Valid default create flags are NC_64BIT_OFFSET, NC_CLOBBER,
+   NC_LOCK, NC_SHARE. */
+int
+nc_set_default_format(int format, int *old_formatp)
+{
+    return lnc_set_default_format(format, old_formatp);
+}
 
 int
 nc__open(const char * path, int ioflags,
@@ -162,8 +176,8 @@ nc__open_mp(const char * urlpath, int ioflags, int basepe,
 	path = path.substr(0, q_mark);
     }
 
-    ClientParams cp(path);
-    NCConnect *c = new NCConnect(cp.url_without_params(), cp);
+    ClientParams *cp = new ClientParams(path);
+    NCConnect *c = new NCConnect(cp->url_without_params(), cp);
 
 #ifdef LOCAL
     if (c->is_local()) {  
@@ -287,8 +301,8 @@ nc_delete(const char * path)
 int
 nc_delete_mp(const char * path, int basepe)
 {
-    ClientParams cp(path);
-    NCConnect *c = new NCConnect(cp.url_without_params(), cp);
+    ClientParams *cp = new ClientParams(path);
+    NCConnect *c = new NCConnect(cp->url_without_params(), cp);
 
 #ifdef LOCAL
     if(c->is_local()) {  
@@ -296,7 +310,8 @@ nc_delete_mp(const char * path, int basepe)
       return rcode;
     }
 #endif
-
+    
+    delete c; c = 0;
     return NC_EPERM;
 }
 
@@ -580,7 +595,7 @@ const char *
 nc_inq_libvers(void)
 {
 #ifdef LOCAL
-  return lnc_inq_libvers(); // local netCDF library version (not DODS)
+  return lnc_inq_libvers(); // local netCDF library version (not OPeNDAP)
 #else
   return ((char *)0);
 #endif
@@ -596,160 +611,33 @@ nc_strerror(int err)
   return((char *)0);
 }
 
-// $Log: Dnc.cc,v $
-// Revision 1.25  2005/06/03 16:34:27  jimg
-// Added instrumentation using to find bugs found while working on bug 773.
-//
-// Revision 1.24  2005/03/31 00:04:51  jimg
-// Modified to use the factory class in libdap++ 3.5.
-//
-// Revision 1.23  2005/03/04 18:10:49  jimg
-// At this point valgrind runs the Unidata tests for both local and remote access
-// and shows no errors or leaks. There are 8 bytes still reachable from an
-// exception, but that's it.
-//
-// Revision 1.22  2005/03/02 17:51:50  jimg
-// Considerable reduction in memory leaks and fixed all errant memory
-// accesses found with nc_test. OPeNDAP error codes and Error object
-// message strings are now reported using the nc_strerrror() function!
-//
-// Revision 1.21  2004/10/28 16:38:19  jimg
-// Added support for error handling to ClientParams. Added use of
-// ClientParams to NCConnect, although that's not complete yet. NCConnect
-// now has an instance of ClientParams. The instance is first built and
-// then passed into NCConnect's ctor which stores a const reference to the CP
-// object.
-//
-// Revision 1.20  2004/09/08 22:08:21  jimg
-// More Massive changes: Code moved from the files that clone the netCDF
-// function calls into NCConnect, NCAccess or nc_util.cc. Much of the
-// translation functions are now methods. The netCDF type classes now
-// inherit from NCAccess in addition to the DAP type classes.
-//
-// Revision 1.19  2004/08/03 23:23:13  jimg
-// Vast changes to get attribute support working for variables from
-// data sources with Structures. The Structure variables are 'translated'
-// to sets of variables with 'dots in their names.' These new changes
-// correctly find the attribtues for the old variables and add them to
-// the built in attribute table objects for the new variables. Global
-// attributes are now handled as well. This software relies heavily on the
-// heuristic code in DDS::transfer_attributes(). During the course of the
-// work, I moved all of the software that interacts with the OPeNDAP servers
-// to NCConnect or nc_util. The files Dvar.cc, Dattr.cc, ..., are now
-// just the clones of the netCDF api functions.
-//
-// Revision 1.18  2004/07/28 18:10:05  jimg
-// Fixed handling of global attributes. I used the new code in
-// DDS::transfer_attributes().
-//
-// Revision 1.17  2004/07/26 19:10:44  jimg
-// Moved netCDF CL <--> OPeNDAP server interface code to nc_util and
-// NCConnect.
-//
-// Revision 1.16  2004/03/09 22:56:32  jimg
-// Refactored so that Pix is no longer used. Some common code (in the
-// glue routines) was also factored out to functions. The netCDF 2
-// interface is now supplied by the lnetcdf/lv2i.c file (which is a mostly
-// unaltered copy of the original v2i.c source file). See lnetcdf/README.
-//
-// Revision 1.15  2004/03/08 19:08:33  jimg
-// This version of the code uses the Unidata netCDF 3.5.1 version of the
-// netCDF 2 API emulation. This functions call our netCDF 3 API functions
-// which may either interact with a DAP server r call the local netCDF 3
-// functions.
-//
-// Revision 1.14  2004/03/01 22:30:58  jimg
-// Update; more fixes for translation of Structure types. This code can
-// be built with ncdump and that client can read from a host of HDF4
-// datasets. The changes that enable this are filtering of Sequences.
-//
-// Revision 1.13  2004/02/25 00:47:52  jimg
-// This code will translate Structures, including ones that are nested.
-// Not tested much; needs work.
-//
-// Revision 1.12  2004/02/19 19:42:55  jimg
-// Merged with release-3-4-2FCS and resolved conflicts.
-//
-// Revision 1.11  2003/12/08 18:06:37  edavis
-// Merge release-3-4 into trunk
-//
-// Revision 1.8.2.4  2003/10/09 05:59:03  reza
-// Fixed for Fortran compiler.
-//
-// Revision 1.10  2003/09/29 12:11:13  reza
-// Merged v3.2 translation code.
-//
-// Revision 1.9  2003/09/25 23:09:36  jimg
-// Meerged from 3.4.1.
-//
-// Revision 1.8.2.3  2003/07/21 07:05:48  reza
-// Added support for Cray multi-processor (CRAYMPP) interfaces, they use processor element 0 for all non-CrayMPP platforms. The Fortran interface needs them for a clean build.
-//
-// Revision 1.8.2.2  2003/06/06 08:23:41  reza
-// Updated the servers to netCDF-3 and fixed error handling between client and server.
-//
-// Revision 1.8.2.1  2003/05/06 13:23:54  reza
-// Fixed the netCDF V3.5 upgrade.
-//
-// Revision 1.8  2003/05/02 16:30:29  jimg
-// Fixes for the builds
-//
-// Revision 1.7  2003/01/28 07:08:24  jimg
-// Merged with release-3-2-8.
-//
-// Revision 1.4.4.9  2002/12/27 00:37:36  jimg
-// Removed some unused variables.
-// Switched to set_ncid() where appropriate.
-//
-// Revision 1.4.4.8  2002/12/23 23:06:40  jimg
-// Build Fixes.
-//
-// Revision 1.4.4.7  2002/11/21 23:02:19  pwest
-// fixed some memory leaks and memory read errors
-//
-// Revision 1.4.4.6  2002/11/06 22:53:23  pwest
-// Cleaned up some uninitialized memory read warnings from purify
-//
-// Revision 1.6  2002/05/03 00:01:52  jimg
-// Merged with release-3-2-7.
-//
-// Revision 1.4.4.5  2002/04/17 20:11:32  jimg
-// Fixed the return value of ncopen() so that any failure results in a return
-// value of -1. This fixes bug 393.
-//
-// Revision 1.4.4.4  2001/12/26 05:53:53  rmorris
-// Force a return value in nc_strerror().  VC++ always requires a return;
-//
-// Revision 1.4.4.3  2001/12/26 03:39:05  rmorris
-// nc_inq_libvers() must return _something_ under win32.  No return
-// never allowed.
-//
-// Revision 1.5  2001/08/30 23:08:23  jimg
-// Merged with 3.2.4
-//
-// Revision 1.4.4.2  2001/08/30 06:23:03  reza
-// Fixed bugs reported by bugzilla.
-//
-// Revision 1.4.4.1  2001/05/17 16:33:57  reza
-// Fixed exception handlers to return correct error codes.
-//
-// Revision 1.4  2000/10/06 01:22:02  jimg
-// Moved the CVS Log entries to the ends of files.
-// Modified the read() methods to match the new definition in the dap library.
-// Added exception handlers in various places to catch exceptions thrown
-// by the dap library.
-//
-// Revision 1.3  1999/11/05 05:15:04  jimg
-// Result of merge woth 3-1-0
-//
-// Revision 1.1.2.2  1999/10/29 05:05:19  jimg
-// Reza's fixes plus the configure & Makefile update
-//
-// Revision 1.2  1999/10/21 13:19:06  reza
-// IMAP and other bug fixed for version3.
-//
-// Revision 1.1.2.1  1999/10/15 19:50:55  jimg
-// Changed return values and conditions for NC API entry points
-//
-// Revision 1.1  1999/07/28 00:22:41  jimg
-// Added
+int
+nc_inq_format(int ncid, int *formatp)
+{
+    if (!conns || ncid < 0 || ncid > MAX_NC_OPEN 
+	|| (*conns)[ncid] == NULL) // does the file id exist?
+	return NC_EBADID;
+
+    if((*conns)[ncid] == NULL) // does the file id exist?
+      return NC_EBADID ;
+
+#ifdef LOCAL
+    if((*conns)[ncid]->is_local()) {
+	rcode = lnc_inq_format((*conns)[ncid]->get_ncid(), formatp);
+      return rcode;
+    }
+#endif
+    // The new (3.6.2 beta 4) from Unidata looks at the NC_64BIT_OFFSET flag
+    // and sets formatp to NC_FORMAT_64BIT if it's true, otherwise is sets it
+    // to NC_FORMAT_CLASSIC. I'm just going to return 64BIT for now since I
+    // think that building the library as 64BIT means everything in memory is
+    // represented using the new format. This probably only matters for
+    // creating files anyway, in which case only the local code is critical
+    // and that's still using the 'real' netcdf library. jhrg 11/01/06
+#if 0
+    *formatp = fIsSet(ncp->flags, NC_64BIT_OFFSET) ? NC_FORMAT_64BIT 
+	: NC_FORMAT_CLASSIC; 
+#endif
+    *formatp = NC_FORMAT_64BIT;
+    return NC_NOERR;
+}
