@@ -40,7 +40,7 @@
 
 #include "config_nc.h"
 
-static char rcsid[] not_used ={"$Id: Dputget.cc 11906 2005-08-08 19:51:43Z root $"};
+static char rcsid[] not_used ={"$Id: Dputget.cc 15133 2006-11-22 21:02:06Z jimg $"};
 
 #include <sstream>
 
@@ -55,8 +55,12 @@ static char rcsid[] not_used ={"$Id: Dputget.cc 11906 2005-08-08 19:51:43Z root
 // Undefine SIZEOF_SIZE_T to force macros in Dv2i.h to expand a certain way.
 // This was actually always happening, but switching to a newer autoconf
 // makes the problem explict. The real fix is to upgrade to netcdf 3.6.1.
-// 5/11/05 jhrg 
+// 5/11/05 jhrg
+#ifdef WIN32
+#define SIZEOF_SIZE_T -1
+#else
 #undef SIZEOF_SIZE_T
+#endif
 #include "Dv2i.h"
 #include "nc_util.h"
 
@@ -1454,14 +1458,15 @@ nc_get_varm_double(int cdfid, int varid,
 // Used for the netCDF 2 interface emulation. See lv2i.c. 
 extern "C" int
 nc_get_varm (
-	     int cdfid,
+	     int ncid,
 	     int varid,
 	     const size_t * start,
 	     const size_t * edges,
 	     const ptrdiff_t * stride,
-	     const ptrdiff_t * imap,
+	     const ptrdiff_t * imapp,
 	     void *value)
 {   
+#if 0
     if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
@@ -1471,10 +1476,105 @@ nc_get_varm (
 	return lnc_get_varm((*conns)[cdfid]->get_ncid(), varid, start,
 			    edges, stride, imap, value);
 #endif
+#endif
 
+#if 0
     return GenRead(cdfid, varid, start, edges, 
 		   stride, imap, (void *)value, Tvoid);
+#else
+    // Unidata's code from 3.6.2 (and earlier?). Since this is written in 
+    // terms of other netcdf functions, I used it here. Starting with netcdf
+    // 3.6.2 the implementation that called GenRead() (above) failed. jhrg
+    // 11/22/06
+    
+    int status;
+    nc_type vartype;
+    int varndims;
+    ptrdiff_t *cvtmap = NULL;
+
+    status = nc_inq_vartype(ncid, varid, &vartype); 
+    if(status != NC_NOERR)
+        return status;
+
+    status = nc_inq_varndims(ncid, varid, &varndims); 
+    if(status != NC_NOERR)
+        return status;
+
+    if(imapp != NULL && varndims != 0)
+    {
+        /*
+         * convert map units from bytes to units of sizeof(type)
+         */
+        size_t ii;
+        const ptrdiff_t szof = (ptrdiff_t) nctypelen(vartype);
+        cvtmap = (ptrdiff_t *)calloc(varndims, sizeof(ptrdiff_t));
+        if(cvtmap == NULL)
+            return NC_ENOMEM;
+        for(ii = 0; ii < varndims; ii++)
+        {
+            if(imapp[ii] % szof != 0)   
+            {
+                free(cvtmap);
+                return NC_EINVAL;
+            }
+            cvtmap[ii] = imapp[ii] / szof;
+        }
+        imapp = cvtmap;
+    }
+
+    switch(vartype){
+    case NC_CHAR:
+        status =  nc_get_varm_text(ncid, varid, start, edges,
+            stride, imapp,
+            (char *) value);
+        break;
+    case NC_BYTE:
+        status = nc_get_varm_schar(ncid, varid, start, edges,
+            stride, imapp,
+            (signed char *) value);
+        break;
+    case NC_SHORT:
+        status = nc_get_varm_short(ncid, varid, start, edges,
+            stride, imapp,
+            (short *) value);
+        break;
+    case NC_INT:
+#if (SIZEOF_INT >= X_SIZEOF_INT)
+        status = nc_get_varm_int(ncid, varid, start, edges,
+            stride, imapp,
+            (int *) value);
+#elif SIZEOF_LONG == X_SIZEOF_INT
+        status = nc_get_varm_long(ncid, varid, start, edges,
+            stride, imapp,
+            (long *) value);
+#else
+#error "nc_get_varm implementation"
+#endif
+        break;
+    case NC_FLOAT:
+        status = nc_get_varm_float(ncid, varid, start, edges,
+            stride, imapp,
+            (float *) value);
+        break;
+    case NC_DOUBLE: 
+        status = nc_get_varm_double(ncid, varid, start, edges,
+            stride, imapp,
+            (double *) value);
+        break;
+    default:
+        status = NC_EBADTYPE;
+        break;
+    }
+
+    if(cvtmap != NULL)
+    {
+        free(cvtmap);
+    }
+    return status;
+    
+#endif
 }
+
 int
 nc_get_vars_text (
 		  int ncid,
