@@ -26,9 +26,12 @@
 // Base class for methods common to all the netCDF DAP dat type classes.
 // jhrg 08/31/04
 
+// #define DODS_DEBUG 1
+
 #include "config_nc.h"
 
-static char rcsid[] not_used ={"$Id: NCAccess.cc 11906 2005-08-08 19:51:43Z root $"};
+static char rcsid[] not_used =
+    { "$Id: NCAccess.cc 13725 2006-04-19 18:06:01Z jimg $" };
 
 #include "BaseType.h"
 #include "InternalErr.h"
@@ -40,17 +43,18 @@ static char rcsid[] not_used ={"$Id: NCAccess.cc 11906 2005-08-08 19:51:43Z root
 
 #include "debug.h"
 
-const string spr = "."; // structure rename
+const string spr = ".";         // structure rename
 
 /** Clone this object. This method is used by the NC&lt;type&gt; classes to
     copy fields that are defined by this object. Code should never create
     an object that is only an NCAccess; the class should always be used
     in combination with one of the NetCDF type classes. */
 void
-NCAccess::clone(const NCAccess &nca)
+ NCAccess::clone(const NCAccess & nca)
 {
     d_impl_version = nca.d_impl_version;
     d_translated = nca.d_translated;
+    d_strdim = nca.d_strdim;
 
     if (nca.d_source)
         d_source = nca.d_source->ptr_duplicate();
@@ -67,31 +71,33 @@ NCAccess::clone(const NCAccess &nca)
     @param edges
     @param stride
     @return The number of elements to be returned */
-    
+
 string
-NCAccess::build_constraint(int outtype, const size_t *start,
-            const size_t *edges, const ptrdiff_t *stride) throw(Error)
+    NCAccess::build_constraint(int outtype, const size_t * /*start*/,
+                               const size_t * /*edges*/,
+                               const ptrdiff_t * /*stride*/) throw(Error)
 {
     if (!is_convertable(outtype))
         throw Error(NC_ECHAR, "Character conversion not supported.");
-  
-    switch(dynamic_cast<BaseType*>(this)->type()) {
-      case dods_str_c:
-      case dods_url_c:  
-      case dods_byte_c:
-      case dods_int16_c:
-      case dods_uint16_c:
-      case dods_int32_c:
-      case dods_uint32_c: 
-      case dods_float32_c:
-      case dods_float64_c:
-        return dynamic_cast<BaseType*>(this)->name();
-        
-      default:
+
+    switch (dynamic_cast < BaseType * >(this)->type()) {
+    case dods_str_c:
+    case dods_url_c:
+    case dods_byte_c:
+    case dods_int16_c:
+    case dods_uint16_c:
+    case dods_int32_c:
+    case dods_uint32_c:
+    case dods_float32_c:
+    case dods_float64_c:
+        return dynamic_cast < BaseType * >(this)->name();
+
+    default:
         throw Error(NC_EBADTYPE,
-            string("The netCDF Client Library cannot request variables of type: ")
-            + dynamic_cast<BaseType*>(this)->type_name()
-            + " [from NCAccess::build_constraint()]");
+                    string
+                    ("The netCDF Client Library cannot request variables of type: ")
+                    + dynamic_cast < BaseType * >(this)->type_name()
+                    + " [from NCAccess::build_constraint()]");
     }
 }
 
@@ -112,7 +118,7 @@ NCAccess::build_constraint(int outtype, const size_t *start,
     
     @param proj A string that contains the projection clauses. */
 void
-NCAccess::store_projection(const string &proj)
+ NCAccess::store_projection(const string & /*proj*/)
 {
 }
 
@@ -121,15 +127,17 @@ NCAccess::store_projection(const string &proj)
     
     @param outtype The output type.
     @return True if the variable can be converted, false otherwise. */
-bool
-NCAccess::is_convertable(int outtype)
+bool NCAccess::is_convertable(int outtype)
 {
-    Type intype = dynamic_cast<BaseType*>(this)->type();
+    Type intype = dynamic_cast < BaseType * >(this)->type();
     DBG(cerr << "intype: " << intype << ", outtype: " << outtype << endl);
 
-    if (((outtype == Ttext) && (intype != dods_str_c) &&(intype != dods_url_c))
-        || ((outtype != Ttext) && (intype == dods_str_c) &&(outtype != Tvoid))
-        || ((outtype != Ttext)&&(intype == dods_url_c)&&(outtype != Tvoid)))
+    if (((outtype == Ttext) && (intype != dods_str_c)
+         && (intype != dods_url_c))
+        || ((outtype != Ttext) && (intype == dods_str_c)
+            && (outtype != Tvoid))
+        || ((outtype != Ttext) && (intype == dods_url_c)
+            && (outtype != Tvoid)))
         return false;
     else
         return true;
@@ -138,13 +146,14 @@ NCAccess::is_convertable(int outtype)
 /** Return the netCDF type of this variable. 
     @note This \b must be specialized for all of the atomic types. This
     version throws an InternalErr exception. */
-nc_type
-NCAccess::get_nc_type() throw(InternalErr)
+nc_type NCAccess::get_nc_type()throw(InternalErr)
 {
+    DBG(cerr << "NCAccess::get_nc_type(): should not get here." << endl);
+
     throw InternalErr(__FILE__, __LINE__,
-        string("NetCDF type information requested for a ")
-        + dynamic_cast<BaseType*>(this)->type_name() 
-        + string("; operation not supported."));
+                      string("NetCDF type information requested for a ")
+                      + dynamic_cast < BaseType * >(this)->type_name()
+                      + string("; operation not supported."));
 }
 
 /** Given that the variable has values, extract those, convert to outtype
@@ -154,92 +163,102 @@ NCAccess::get_nc_type() throw(InternalErr)
     
     @param values Buffer for the extracted values.
     @param outtype The desired type for the values.
+    @param cp A reference to teh ClientParams object for this connection. Some
+    of the extract_values specializations use this to control how values are
+    extracted.
     @exception Error Thrown if no data are present, if there's an error 
     copying/converting values/types or if the type of the variable is not
     supported (the latter should never happen). 
     @see NCArray::extract_values() */
-void
-NCAccess::extract_values(void *values, int elements, int outtype) throw(Error)
-{    
-    BaseType *btp = dynamic_cast<BaseType*>(this);
+void NCAccess::extract_values(void *values, int elements, int outtype,
+                              const ClientParams & /* cp */ ) throw(Error)
+{
+    BaseType *btp = dynamic_cast < BaseType * >(this);
+    DBG(cerr << "btp->type() = " << btp->type() << endl);
     switch (btp->type()) {
-      case dods_byte_c:
-      case dods_int16_c:
-      case dods_uint16_c:
-      case dods_int32_c:
-      case dods_uint32_c: 
-      case dods_float32_c:
-      case dods_float64_c: {
-        // If this (apparently) scalar variable is part of a Sequence, then
-        // The netCDF library must think it's an Array. Get the number of 
-        // elements (which is the number of rows in the returned Sequence).
-        // By calling find_ancestral_sequence() the case where this variable
-        // is a member of a structure that's inside a sequence is covered.
-        BaseType &bt = dynamic_cast<BaseType&>(*this);   
-        NCSequence *ncq = dynamic_cast<NCSequence*>(find_ancestral_sequence(&bt));
-        int nels = (ncq) ? ncq->number_of_rows() : 1;
-    
-        // Allocate storage for the values
-        int data_width = bt.width();
-        // Why allocate 'elements' instead of 'nels'? nels is the number of 
-        // elements actually returned; elements is the number the client 
-        // thinks it's getting. We allocate enough to padd with zeros. Then
-        // convert_nc_type is called to transfer the information to the values 
-        // buffer.
+    case dods_byte_c:
+    case dods_int16_c:
+    case dods_uint16_c:
+    case dods_int32_c:
+    case dods_uint32_c:
+    case dods_float32_c:
+    case dods_float64_c:{
+            // If this (apparently) scalar variable is part of a Sequence, then
+            // The netCDF library must think it's an Array. Get the number of 
+            // elements (which is the number of rows in the returned Sequence).
+            // By calling find_ancestral_sequence() the case where this variable
+            // is a member of a structure that's inside a sequence is covered.
+            BaseType & bt = dynamic_cast < BaseType & >(*this);
+            NCSequence *ncq =
+                dynamic_cast <
+                NCSequence * >(find_ancestral_sequence(&bt));
+            int nels = (ncq) ? ncq->number_of_rows() : 1;
+
+            // Allocate storage for the values
+            int data_width = bt.width();
+            // Why allocate 'elements' instead of 'nels'? nels is the number of 
+            // elements actually returned; elements is the number the client 
+            // thinks it's getting. We allocate enough to padd with zeros. Then
+            // convert_nc_type is called to transfer the information to the values 
+            // buffer.
 #if 0
-        void *tmpbufin = new char[nels * data_width];
+            void *tmpbufin = new char[nels * data_width];
 #endif
-        char *tmpbufin = new char[elements * data_width];
-        int bytes = 0;
-    
-        if (ncq) {
-            char *tptr = tmpbufin;
-            int i = 0;
-            while (i < nels) {
-                // Using an index in place of bt.name() would be faster
-                bytes += ncq->var_value(i, bt.name())->buf2val((void**)&tptr);
-                tptr += data_width;
-                ++i;
-            }
-            // Pad tptr so any values not read are zeros
-            while (i < elements) {
-                for (int j = 0; j < data_width; ++j)
-                    *tptr++ = 0;
-                ++i;                
+            char *tmpbufin = new char[elements * data_width];
+            int bytes = 0;
+
+            if (ncq) {
+                char *tptr = tmpbufin;
+                int i = 0;
+                while (i < nels) {
+                    // Using an index in place of bt.name() would be faster
+                    bytes +=
+                        ncq->var_value(i,
+                                       bt.name())->
+                        buf2val((void **) &tptr);
+                    tptr += data_width;
+                    ++i;
+                }
+                // Pad tptr so any values not read are zeros
+                while (i < elements) {
+                    for (int j = 0; j < data_width; ++j)
+                        *tptr++ = 0;
+                    ++i;
+                }
+            } else {
+                bytes = bt.buf2val((void **) &tmpbufin);
             }
+
+            if (bytes == 0)
+                throw Error(-1,
+                            "Could not read any data from remote server.");
+
+            // Get the netCDF type code for this variable.
+            nc_type typep =
+                dynamic_cast < NCAccess * >(this)->get_nc_type();
+
+            int rcode =
+                convert_nc_type(typep, outtype, elements, tmpbufin,
+                                values);
+            delete[]tmpbufin;
+            tmpbufin = 0;
+            if (rcode != NC_NOERR)
+                throw Error(rcode,
+                            "Error copying values between internal buffers [NCAccess::extract_values()]");
+            break;
         }
-        else {
-            bytes = bt.buf2val((void**)&tmpbufin);
-        }
-        
-        if (bytes == 0)
-            throw Error(-1, "Could not read any data from remote server.");
-    
-        // Get the netCDF type code for this variable.
-        nc_type typep = dynamic_cast<NCAccess*>(this)->get_nc_type();
-    
-        int rcode = convert_nc_type(typep, outtype, elements, tmpbufin, values);
-        delete[] tmpbufin; tmpbufin = 0;
-        if (rcode != NC_NOERR)
-            throw Error(rcode,
-                "Error copying values between internal buffers [NCAccess::extract_values()]");
-#if 0
-        if (tmpbufin != values)
-            delete tmpbufin;
-#endif
-        break;
-      }
-      
-      case dods_str_c:
-      case dods_url_c:
+
+    case dods_str_c:
+    case dods_url_c:
         throw InternalErr(__FILE__, __LINE__, "Should never get here!!");
-   
-      default:
+
+    default:
         throw Error(NC_EBADTYPE,
-            string("The netCDF Client Library cannot access variables of type: ")
-            + dynamic_cast<BaseType*>(this)->type_name()
-            + " [NCAccess::extract_values()]");
-    } // End of the switch
+                    string
+                    ("The netCDF Client Library cannot access variables of type: ")
+                    + btp->type_name()
+                    + " [NCAccess::extract_values()]");
+    }                           // End of the switch
 }
 
 /** If this variable was created during a translation operation, return the
@@ -249,8 +268,7 @@ NCAccess::extract_values(void *values, int elements, int outtype) throw(Error)
     
     @return The source variable if this was translated, or null if not
     translated. */
-BaseType *
-NCAccess::get_source() const
+BaseType *NCAccess::get_source() const
 {
     return d_source;
 }
@@ -259,8 +277,7 @@ NCAccess::get_source() const
     @param s A pointer to this variable's source variable.
     @exception InternalErr Thrown by default; overload this in NCArray, 
     et cetera. */
-void
-NCAccess::set_source(BaseType *s) throw(InternalErr)
+void NCAccess::set_source(BaseType * /*s*/) throw(InternalErr)
 {
     throw InternalErr(__FILE__, __LINE__, "Unimplemented method");
 }
@@ -283,7 +300,7 @@ NCAccess::set_source(BaseType *s) throw(InternalErr)
     @param cp A const reference to the ClientParams object.
     @return A VarList that holds all of the 'flattened' variables. */
 VarList
-NCAccess::flatten(const ClientParams &cp, const string &parent_name)
+    NCAccess::flatten(const ClientParams & /*cp*/, const string & parent_name)
 {
     VarList new_vars;
 
@@ -291,29 +308,24 @@ NCAccess::flatten(const ClientParams &cp, const string &parent_name)
     // instance and ptr_duplicate() arranges to copy it, using ptr_duplicate()
     // also copies the attributes of the variable referenced by *ctor to
     // the new variable. jhrg 10/26/04
-    BaseType *source = dynamic_cast<BaseType*>(this);
-    if (!source)
-        throw InternalErr(__FILE__, __LINE__, "Instance of NCAccess not a BaseType!");
-    BaseType *btp = source->ptr_duplicate();
-#if 0
-    BaseType *btp = dynamic_cast<BaseType&>(*this).ptr_duplicate();
-#endif    
+    BaseType *btp = dynamic_cast < BaseType & >(*this).ptr_duplicate();
+
     if (!parent_name.empty()) {
-        btp->set_name(parent_name + spr + dynamic_cast<BaseType&>(*this).name());
+        btp->set_name(parent_name + spr + dynamic_cast <
+                      BaseType & >(*this).name());
         btp->get_attr_table().append_attr("translation", "String",
                                           "\"flatten\"");
-        dynamic_cast<NCAccess&>(*btp).set_translated(true);
-        DBG2(cerr << "Marking " << btp->name() << " (" << btp 
-                 << ") as translated." << endl);
+        dynamic_cast < NCAccess & >(*btp).set_translated(true);
+        DBG2(cerr << "Marking " << btp->name() << " (" << btp
+             << ") as translated." << endl);
     }
 
-    new_vars.push_back(btp); 
+    new_vars.push_back(btp);
 
     return new_vars;
 }
 
-BaseType *
-NCAccess::find_child_sequence()
+BaseType *NCAccess::find_child_sequence()
 {
     return 0;
 }
