@@ -53,13 +53,14 @@ using namespace std;
 class NCAccess {
     string d_impl_version;
 
-    NCAccess &operator=(const NCAccess &rhs) {
+    NCAccess &operator=(const NCAccess &/*rhs*/) {
         throw InternalErr(__FILE__, __LINE__, "Assignment not implemented"); 
     }    
 
 protected:
     BaseType *d_source;       /// Reference to source var if translated
     bool d_translated;
+    size_t d_strdim;
     
 public:
     NCAccess() : d_impl_version(""), d_source(0), d_translated(false) {}
@@ -86,8 +87,8 @@ public:
     virtual nc_type get_nc_type() throw(InternalErr);
     
     /// Read data from the variable.
-    virtual void extract_values(void *values, int elements, int outtype)
-        throw(Error);
+    virtual void extract_values(void *values, int elements, int outtype,
+                                const ClientParams &cp) throw(Error);
     
     /// If created during translation, return the source variable.
     virtual BaseType *get_source() const;
@@ -111,6 +112,15 @@ public:
     /// Set the \e translated property.
     virtual void set_translated(const bool t) { d_translated = t; }
     
+
+    /// set the string dimension
+    virtual void set_strdim(size_t strdim) { d_strdim = strdim; }
+
+
+    /// get the string dimension
+    virtual size_t get_strdim(void) { return d_strdim; }
+
+
     /// Flatten this variable.
     virtual VarList flatten(const ClientParams &cp, const string &parent_name);
     
