@@ -38,7 +38,8 @@
 
 #include "config_nc.h"
 
-static char rcsid[] not_used = {"$Id: NCArray.cc 11906 2005-08-08 19:51:43Z root $"};
+static char rcsid[] not_used =
+    { "$Id: NCArray.cc 13725 2006-04-19 18:06:01Z jimg $" };
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -61,10 +62,9 @@ static char rcsid[] not_used = {"$Id: NCArray.cc 11906 2005-08-08 19:51:43Z root
 #include "nc_util.h"
 #include "debug.h"
 
-const string spr = "."; // structure rename
+const string spr = ".";         // structure rename
 
-BaseType *
-NCArray::ptr_duplicate()
+BaseType *NCArray::ptr_duplicate()
 {
     return new NCArray(*this);
 }
@@ -76,11 +76,11 @@ NCArray::ptr_duplicate()
     storage used by the actual parameter. Also, if the actual parameter
     is an Array, libdap++ code will use the template of that Array as the
     template for this NCArray. */
-NCArray::NCArray(const string &n, BaseType *v) : Array(n, v)
+NCArray::NCArray(const string & n, BaseType * v):Array(n, v)
 {
 }
 
-NCArray::NCArray(const NCArray &rhs) : Array(rhs)
+NCArray::NCArray(const NCArray & rhs): Array(rhs), NCAccess(rhs)
 {
     m_duplicate(rhs);
 }
@@ -88,22 +88,23 @@ NCArray::NCArray(const NCArray &rhs) : Array(rhs)
 NCArray::~NCArray()
 {
     DBG(cerr << "delete d_source " << d_source << endl);
-    delete d_source; d_source = 0;
+    delete d_source;
+    d_source = 0;
 }
 
 void
-NCArray::m_duplicate(const NCArray &nca)
+ NCArray::m_duplicate(const NCArray & nca)
 {
-    dynamic_cast<NCAccess&>(*this).clone(dynamic_cast<const NCAccess&>(nca));
+    dynamic_cast < NCAccess & >(*this).clone(dynamic_cast <
+                                             const NCAccess & >(nca));
 }
 
-NCArray &
-NCArray::operator=(const NCArray &rhs)
+NCArray & NCArray::operator=(const NCArray & rhs)
 {
     if (this == &rhs)
         return *this;
 
-    dynamic_cast<Array &>(*this) = rhs;
+    dynamic_cast < Array & >(*this) = rhs;
 
     m_duplicate(rhs);
 
@@ -112,9 +113,8 @@ NCArray::operator=(const NCArray &rhs)
 
 
 // Should this be a private method? jhrg 11/3/04
-long
-NCArray::format_constraint(size_t *cor, ptrdiff_t *step, size_t *edg,
-                           bool *has_stride)
+long NCArray::format_constraint(size_t * cor, ptrdiff_t * step,
+                                size_t * edg, bool * has_stride)
 {
     int start, stride, stop;
     int id = 0;
@@ -123,40 +123,39 @@ NCArray::format_constraint(size_t *cor, ptrdiff_t *step, size_t *edg,
     *has_stride = false;
 
     for (Dim_iter p = dim_begin(); p != dim_end(); ++p, id++) {
-    	start = dimension_start(p, true); 
-    	stride = dimension_stride(p, true);
-    	stop = dimension_stop(p, true);
-    	// Check for empty constraint
-    	if (start + stop + stride == 0)
-    	    return -1;
-    
-    	cor[id] = start;
-    	step[id] = stride;
-    	edg[id] = ((stop - start)/stride) + 1; // count of elements
-    	nels *= edg[id];      // total number of values for variable
-    
-    	if (stride != 1)
-    	    *has_stride = true;
+        start = dimension_start(p, true);
+        stride = dimension_stride(p, true);
+        stop = dimension_stop(p, true);
+        // Check for empty constraint
+        if (start + stop + stride == 0)
+            return -1;
+
+        cor[id] = start;
+        step[id] = stride;
+        edg[id] = ((stop - start) / stride) + 1;        // count of elements
+        nels *= edg[id];        // total number of values for variable
+
+        if (stride != 1)
+            *has_stride = true;
     }
-    
+
     return nels;
 }
 
 
-void
-NCArray::store_projection(const string &proj)
+void NCArray::store_projection(const string & proj)
 {
     // Scan the projections looking for one about this variable
     string::size_type name_pos = proj.find(name());
     if (name_pos == string::npos)
         return;
-    
+
     // OK, this variable is in there. At name_pos
     string clause;
     string::size_type end_clause_pos = proj.find(',', name_pos);
     if (end_clause_pos == string::npos)
         clause = proj.substr(name_pos);
-    else 
+    else
         clause = proj.substr(name_pos, end_clause_pos - name_pos);
 
     // For each bracketed group of numbers, get information about a
@@ -165,21 +164,23 @@ NCArray::store_projection(const string &proj)
     string::size_type clause_start = clause.find("[", idx);
     string::size_type clause_end = clause.find("]", idx);
     while (clause_start != string::npos || clause_end != string::npos) {
-        string dim_proj = clause.substr(clause_start, clause_end - clause_start + 1);
+        string dim_proj =
+            clause.substr(clause_start, clause_end - clause_start + 1);
 
         struct dim_proj_info dpi(dim_proj);
         cl_proj.push_back(dpi);
-        
+
         idx = clause_end + 1;
         clause_start = clause.find("[", idx);
         clause_end = clause.find("]", idx);
     }
-        
+
 }
 
 string
-NCArray::build_constraint(int outtype, const size_t *start,
-            const size_t *edges, const ptrdiff_t *stride) throw(Error)
+    NCArray::build_constraint(int outtype, const size_t * start,
+                              const size_t * edges,
+                              const ptrdiff_t * stride) throw(Error)
 {
     // Test for bad conversion here, before we may call NCSequence (which
     // cannot run the test).
@@ -191,26 +192,27 @@ NCArray::build_constraint(int outtype, const size_t *start,
     string expr = name();
 
     // Get dimension sizes and strings for constraint hyperslab
-    ostringstream ce;       // Build CE here.
-    int dm = 0;             // Index to start, edge, stride arrays 
-    int Zedge = 0;          // Search for zero size edges (no data)
+    ostringstream ce;           // Build CE here.
+    int dm = 0;                 // Index to start, edge, stride arrays 
+    int Zedge = 0;              // Search for zero size edges (no data)
 
     // If we found an array...
     Array::Dim_iter d;
-    vector<dim_proj_info>::iterator cl_iter = cl_proj.begin();
+    vector < dim_proj_info >::iterator cl_iter = cl_proj.begin();
     for (d = dim_begin(); d != dim_end(); ++d, ++dm) {
         // Verify stride argument.
         if (stride != NULL && stride[dm] < 1)
             throw Error(NC_ESTRIDE, "Stride less than 1 (one).");
-    
+
         // Set defaults:
         // *start         NULL => first corner 
         // *edges         NULL => everything following start[] 
         // *stride        NULL => unity strides 
         long instart = start != NULL ? start[dm] : 0;
-        long inedges = edges != NULL ? edges[dm] : dimension_size(d) - instart;
+        long inedges =
+            edges != NULL ? edges[dm] : dimension_size(d) - instart;
         long instep = stride != NULL ? stride[dm] : 1;
- 
+
         int ext_start;
         int ext_step;
         int ext_stop;
@@ -219,53 +221,59 @@ NCArray::build_constraint(int outtype, const size_t *start,
             ext_step = (*cl_iter).stride;
             ext_stop = (*cl_iter).stop;
             ++cl_iter;
-        }
-        else {
+        } else {
             ext_start = 0;
             ext_step = 1;
             ext_stop = dimension_size(d) - 1;
         }
 
-        DBG2(cerr << instart <<" "<< inedges <<" "<< dm << endl);
-        DBG2(cerr<< ext_start <<" "<< ext_step <<" " << ext_stop << endl);
-    
+        DBG2(cerr << instart << " " << inedges << " " << dm << endl);
+        DBG2(cerr << ext_start << " " << ext_step << " " << ext_stop <<
+             endl);
+
         // create constraint expr. by combining the constraint from the
         // command line with the constraint gleaned from the API call.
         int Tstart = ext_start + instart * ext_step;
         int Tstep = instep * ext_step;
-        int Tstop = (ext_stop < ext_start+(instart+(inedges-1)*instep)*ext_step) 
-                    ? ext_stop : ext_start+(instart+(inedges-1)*instep)*ext_step;
-    
+        int Tstop =
+            (ext_stop <
+             ext_start + (instart + (inedges - 1) * instep) * ext_step)
+            ? ext_stop : ext_start + (instart +
+                                      (inedges - 1) * instep) * ext_step;
+
         // Check the validity of the array constraints
         if ((instart >= dimension_size(d))
-            || (instart < 0)||(inedges < 0))
-            throw Error(NC_EINVALCOORDS, "Invalid coordinates.");      
-    
+            || (instart < 0) || (inedges < 0))
+            throw Error(NC_EINVALCOORDS, "Invalid coordinates.");
+
         if (instart + inedges > dimension_size(d))
-            throw Error(NC_EEDGE, "Hyperslab size exceeds dimension size.");
-    
+            throw Error(NC_EEDGE,
+                        "Hyperslab size exceeds dimension size.");
+
         // Zero edge found, but first check remaining dimensions for error
-        if (inedges == 0) 
-            Zedge = 1; 
-    
+        if (inedges == 0)
+            Zedge = 1;
+
         ce << "[" << Tstart << ":" << Tstep << ":" << Tstop << "]";
     }
 
     if (Zedge == 1)
         throw Error(NC_NOERR, "The constraint would return no data.");
 
-    expr += ce.str();     // Return ce via value-result parameter.
+    expr += ce.str();           // Return ce via value-result parameter.
     return expr;
 }
 
-bool
-NCArray::is_convertable(int outtype)
+bool NCArray::is_convertable(int outtype)
 {
     Type intype = var()->type();
-    
-    if (((outtype == Ttext) && (intype != dods_str_c) &&(intype != dods_url_c))
-        || ((outtype != Ttext) && (intype == dods_str_c) &&(outtype != Tvoid))
-        || ((outtype != Ttext)&&(intype == dods_url_c)&&(outtype != Tvoid)))
+
+    if (((outtype == Ttext) && (intype != dods_str_c)
+         && (intype != dods_url_c))
+        || ((outtype != Ttext) && (intype == dods_str_c)
+            && (outtype != Tvoid))
+        || ((outtype != Ttext) && (intype == dods_url_c)
+            && (outtype != Tvoid)))
         return false;
     else
         return true;
@@ -281,122 +289,84 @@ NCArray::is_convertable(int outtype)
     @param values Destination buffer
     @param elements Expect this number of elements (i.e., what values can hold)
     @param outtype The netCDF type the client expects to see. */
-void
-NCArray::extract_values(void *values, int elements, int outtype) throw(Error)
+void NCArray::extract_values(void *values, int /*elements*/, int outtype,
+                             const ClientParams & cp) throw(Error)
 {
-    int nels  = length();       // number of elements
-    
+    int nels = length();        // number of elements
+
     switch (var()->type()) {
-      case dods_byte_c:
-      case dods_int16_c:
-      case dods_uint16_c:
-      case dods_int32_c:
-      case dods_uint32_c: 
-      case dods_float32_c:
-      case dods_float64_c: {
-        char *tmpbufin = 0;
-        int bytes = buf2val((void **)&tmpbufin); 
-        if (bytes == 0)
-            throw Error(-1, "Could not read any data from remote server.");
-    
-        // Get the netCDF type code for this variable.
-        nc_type typep = dynamic_cast<NCAccess*>(var())->get_nc_type();
-        int rcode = convert_nc_type(typep, outtype, nels, tmpbufin, values);
-        delete[] tmpbufin; tmpbufin = 0;
-        if (rcode != NC_NOERR)
-            throw Error(rcode, "Error copying values between internal buffers [NCArray::extract_values()]");
-            
-        break;
-      } // End of the case where the array template type is a byte, ..., float64
+    case dods_byte_c:
+    case dods_int16_c:
+    case dods_uint16_c:
+    case dods_int32_c:
+    case dods_uint32_c:
+    case dods_float32_c:
+    case dods_float64_c:{
+            char *tmpbufin = 0;
+            int bytes = buf2val((void **) &tmpbufin);
+            if (bytes == 0)
+                throw Error(-1,
+                            "Could not read any data from remote server.");
+
+            // Get the netCDF type code for this variable.
+            nc_type typep =
+                dynamic_cast < NCAccess * >(var())->get_nc_type();
+            int rcode =
+                convert_nc_type(typep, outtype, nels, tmpbufin, values);
+            delete[]tmpbufin;
+            tmpbufin = 0;
+            if (rcode != NC_NOERR)
+                throw Error(rcode,
+                            "Error copying values between internal buffers [NCArray::extract_values()]");
+
+            break;
+        }                       // End of the case where the array template type is a byte, ..., float64
+
+    case dods_str_c:
+    case dods_url_c:{
+            // rcode = NC_NOERR;
+            char *tbfr = (char *) values;
+            string *sa = new string[nels];
+            string **spa = &sa;
+            buf2val((void **) spa);
+
+            DBG(cerr << "nels: " << nels << endl);
+            DBG(cerr << "strdim: " << strdim << endl);
+            DBG(for (int i = 0; i < nels; ++i) cerr << sa[i] << endl);
 
-      case dods_str_c:
-      case dods_url_c: {
-        // rcode = NC_NOERR;
-        char * tbfr = (char *)values;
-        string *sa = new string[nels];
-        string **spa = &sa;
-        buf2val((void **)spa);
-        
-        DBG(cerr << "nels: " << nels << endl);
-        DBG(for (int i = 0; i < nels; ++i) cerr << sa[i] << endl);
-        
-        // If this string is held in a translated variable and STRINGS_AS_ARRAY
-        // is defined, then the netCDF API has told the client this DAP String
-        // is an STRING_ARRAY_SIZE array. Thus the values buffer is 
-        // nels * STRING_ARRAY_SIZE bytes. When transferring information to 
-        // this buffer, the code must not transfer more than STRING_ARRAY_SIZE
-        // bytes and must pad to that size for individual elements that are
-        // shorter. jhrg 2/24/05
-#ifdef STRING_AS_ARRAY
-        if (get_translated()) {
             for (int i = 0; i < nels; i++) {
-                unsigned int c = 0;
-                while (c < STRING_ARRAY_SIZE 
-                       && (c < sa[i].length()
-                           || (sa[i].length() == 0 && c == 0))) {
-                    *tbfr++ = *(sa[i].c_str() + c++);
-                }
-                while (c++ < STRING_ARRAY_SIZE) {
-                    *tbfr++ = 0;
-                }
-            }
-        }
-        else {
-#endif
-            for (int i = 0; i < nels; i++) {
-                for (unsigned int c = 0;
+                unsigned int c;
+                for (c = 0;
                      c < sa[i].length() || (sa[i].length() == 0 && c == 0);
                      c++) {
-                    // I think it may be a bug to transfer more than one char since
-                    // the client program has allocated a nels buffer for the data.
-                    // jhrg 2/24/05
                     *tbfr++ = *(sa[i].c_str() + c);
                 }
+                if (cp.get_string_as_char_array()) {
+                    unsigned int strdim = get_strdim();
+                    while (c++ < strdim)
+                        *tbfr++ = 0;
+                }
             }
-#ifdef STRING_AS_ARRAY
-        }       // Closes the 'else' above
-#endif
+            delete[]sa;
+            break;
+        }                       // End of the case where the array template type is a string or url
 
-        delete[] sa;       
-#if 0
-        // This code works with the 3.4 libdap++
-        for (int i = 0; i < nels; i++) {
-            Str *s = dynamic_cast<Str*>(var(i));
-            if (!s)
-                throw InternalErr(__FILE__, __LINE__, "Bad cast to Str.");
-            string *cp = 0;
-            string **cpp = &cp;
-            s->buf2val((void **)cpp);
-     
-            for (unsigned int cntr=0; cntr<cp->length() || 
-                 (cp->length()==0 && cntr==0); cntr++) {
-                *tbfr = *(cp->c_str()+cntr);
-                tbfr++;
-            }
-            
-            // Now get rid of the C++ string object.
-            delete cp;
-         }
-#endif
-         break;
-      } // End of the case where the array template type is a string or url
-   
-      default:
+    default:
         throw Error(NC_EBADTYPE,
-            string("The netCDF Client Library cannot access variables of type: ")
-            + dynamic_cast<BaseType*>(this)->type_name()
-            + " [NCArray::extract_values()]");
-    } // End of the switch
+                    string
+                    ("The netCDF Client Library cannot access variables of type: ")
+                    + dynamic_cast < BaseType * >(this)->type_name()
+                    + " [NCArray::extract_values()]");
+    }                           // End of the switch
 }
 
-bool
-NCArray::read(const string &dataset)
+bool NCArray::read(const string & dataset)
 {
     int varid;                  /* variable Id */
     nc_type datatype;           /* variable data type */
-    size_t cor[MAX_NC_DIMS];      /* corner coordinates */
-    size_t edg[MAX_NC_DIMS];      /* edges of hypercube */
-    ptrdiff_t step[MAX_NC_DIMS];     /* stride of hypercube */
+    size_t cor[MAX_NC_DIMS];    /* corner coordinates */
+    size_t edg[MAX_NC_DIMS];    /* edges of hypercube */
+    ptrdiff_t step[MAX_NC_DIMS];        /* stride of hypercube */
     int vdims[MAX_VAR_DIMS];    /* variable dimension sizes */
     int num_dim;                /* number of dim. in variable */
     long nels;                  /* number of elements in buffer */
@@ -418,226 +388,235 @@ NCArray::read(const string &dataset)
 
     DBG(cerr << "In NCArray::read" << endl);
 
-    if (read_p())  // Nothing to do
+    if (read_p())               // Nothing to do
         return false;
 
-    DBG(cerr << "In NCArray, opening the dataset, reading " << name() << endl);
+    DBG(cerr << "In NCArray, opening the dataset, reading " << name() <<
+        endl);
 
     int ncid, errstat;
 
-    errstat = lnc_open(dataset.c_str(), NC_NOWRITE, &ncid); /* netCDF id */
+    errstat = lnc_open(dataset.c_str(), NC_NOWRITE, &ncid);     /* netCDF id */
 
     if (errstat != NC_NOERR)
-      throw Error(errstat, "Could not open the dataset's file.");
- 
+        throw Error(errstat, "Could not open the dataset's file.");
+
     errstat = lnc_inq_varid(ncid, name().c_str(), &varid);
     if (errstat != NC_NOERR)
-      throw Error(errstat, "Could not get variable ID.");
+        throw Error(errstat, "Could not get variable ID.");
 
-    errstat = lnc_inq_var(ncid, varid, (char *)0, &datatype, &num_dim, vdims,
-			(int *)0);
+    errstat =
+        lnc_inq_var(ncid, varid, (char *) 0, &datatype, &num_dim, vdims,
+                    (int *) 0);
     if (errstat != NC_NOERR)
-    throw Error(errstat, 
-		string("Could not read information about the variable `") 
-		+ name() + string("'."));
+        throw Error(errstat,
+                    string
+                    ("Could not read information about the variable `")
+                    + name() + string("'."));
 
     nels = format_constraint(cor, step, edg, &has_stride);
 
     // without constraint (read everything)
-    if ( nels == -1 ){
+    if (nels == -1) {
         nels = 1;
-	has_stride = false;
+        has_stride = false;
         for (id = 0; id < num_dim; id++) {
             cor[id] = 0;
 
-            errstat = lnc_inq_dim(ncid, vdims[id], (char *)0, &vdim_siz);
-	    if (errstat != NC_NOERR)
-	      throw Error(errstat, 
-			  string("Could not inquire dimension information for variable `") 
-			  + name() + string("'."));
+            errstat = lnc_inq_dim(ncid, vdims[id], (char *) 0, &vdim_siz);
+            if (errstat != NC_NOERR)
+                throw Error(errstat,
+                            string
+                            ("Could not inquire dimension information for variable `")
+                            + name() + string("'."));
 
             edg[id] = vdim_siz;
-            nels *= vdim_siz;      /* total number of values for variable */
+            nels *= vdim_siz;   /* total number of values for variable */
         }
     }
-
     // Correct data types to match with the local machine data types
 
-    if ((datatype == NC_FLOAT) 
-	&& (nctypelen(datatype) != sizeof(dods_float32))) {
+    if ((datatype == NC_FLOAT)
+        && (nctypelen(datatype) != sizeof(dods_float32))) {
 
-        fltbuf = (float *) new char [(nels*nctypelen(datatype))];
+        fltbuf = (float *) new char[(nels * nctypelen(datatype))];
 
-	if( has_stride)
-	  errstat = lnc_get_vars_float(ncid, varid, cor, edg, step, fltbuf);
-	else
-	  errstat = lnc_get_vara_float(ncid, varid, cor, edg, fltbuf);
+        if (has_stride)
+            errstat =
+                lnc_get_vars_float(ncid, varid, cor, edg, step, fltbuf);
+        else
+            errstat = lnc_get_vara_float(ncid, varid, cor, edg, fltbuf);
 
-	if (errstat != NC_NOERR)
-	  throw Error(errstat, 
-		      string("Could not read the variable `") + name() 
-		      + string("'."));
+        if (errstat != NC_NOERR)
+            throw Error(errstat,
+                        string("Could not read the variable `") + name()
+                        + string("'."));
 
-	flt32 = new dods_float32 [nels]; 
+        flt32 = new dods_float32[nels];
 
-        for (id = 0; id < nels; id++) 
-            *(flt32+id) = (dods_float32) *(fltbuf+id);
+        for (id = 0; id < nels; id++)
+            *(flt32 + id) = (dods_float32) * (fltbuf + id);
 
-	set_read_p(true);  
-        val2buf((void *)flt32);
+        set_read_p(true);
+        val2buf((void *) flt32);
 
-	// clean up
-        delete [] flt32;
-        delete [] fltbuf;
-    }
-    else if ((datatype == NC_DOUBLE) 
-	&& (nctypelen(datatype) != sizeof(dods_float64))) {
+        // clean up
+        delete[]flt32;
+        delete[]fltbuf;
+    } else if ((datatype == NC_DOUBLE)
+               && (nctypelen(datatype) != sizeof(dods_float64))) {
 
-        dblbuf = (double *) new char [(nels*nctypelen(datatype))];
+        dblbuf = (double *) new char[(nels * nctypelen(datatype))];
 
-	if( has_stride)
-	    errstat = lnc_get_vars_double(ncid, varid, cor, edg, step, dblbuf);
-	else
-	    errstat = lnc_get_vara_double(ncid, varid, cor, edg, dblbuf);
+        if (has_stride)
+            errstat =
+                lnc_get_vars_double(ncid, varid, cor, edg, step, dblbuf);
+        else
+            errstat = lnc_get_vara_double(ncid, varid, cor, edg, dblbuf);
 
-	if (errstat != NC_NOERR)
-	  throw Error(errstat,string("Could not read the variable `") + name() 
-		      + string("'."));
+        if (errstat != NC_NOERR)
+            throw Error(errstat,
+                        string("Could not read the variable `") + name()
+                        + string("'."));
 
-	flt64 = new dods_float64 [nels]; 
+        flt64 = new dods_float64[nels];
 
-        for (id = 0; id < nels; id++) 
-            *(flt64+id) = (dods_float64) *(dblbuf+id);
+        for (id = 0; id < nels; id++)
+            *(flt64 + id) = (dods_float64) * (dblbuf + id);
 
-	set_read_p(true);  
-        val2buf((void *)flt64);
+        set_read_p(true);
+        val2buf((void *) flt64);
 
-	// clean up
-        delete [] flt64;
-        delete [] dblbuf;
-    }
-    else if ((datatype == NC_SHORT) 
-	&& (nctypelen(datatype) != sizeof(dods_int16))) {
+        // clean up
+        delete[]flt64;
+        delete[]dblbuf;
+    } else if ((datatype == NC_SHORT)
+               && (nctypelen(datatype) != sizeof(dods_int16))) {
 
-        shtbuf = (short *)new char [(nels*nctypelen(datatype))];
+        shtbuf = (short *) new char[(nels * nctypelen(datatype))];
 
-	if( has_stride)
-	    errstat = lnc_get_vars_short(ncid, varid, cor, edg, step, shtbuf);
-	else 
-	    errstat = lnc_get_vara_short(ncid, varid, cor, edg, shtbuf);
-	
-	if (errstat != NC_NOERR)
-	  throw Error(errstat, 
-		      string("Could not read the variable `") + name() 
-		      + string("'."));
+        if (has_stride)
+            errstat =
+                lnc_get_vars_short(ncid, varid, cor, edg, step, shtbuf);
+        else
+            errstat = lnc_get_vara_short(ncid, varid, cor, edg, shtbuf);
 
-	intg16 = new dods_int16 [nels];
+        if (errstat != NC_NOERR)
+            throw Error(errstat,
+                        string("Could not read the variable `") + name()
+                        + string("'."));
 
-        for (id = 0; id < nels; id++) 
-            *(intg16+id) = (dods_int16) *(shtbuf+id);
+        intg16 = new dods_int16[nels];
 
-	set_read_p(true);  
-        val2buf((void *)intg16);
+        for (id = 0; id < nels; id++)
+            *(intg16 + id) = (dods_int16) * (shtbuf + id);
 
-	// clean up
-        delete [] intg16;
-        delete [] shtbuf;
-    }
-    else if ((datatype == NC_LONG) 
-	&& (nctypelen(datatype) != sizeof(dods_int32))) {
+        set_read_p(true);
+        val2buf((void *) intg16);
 
-      lngbuf = (nclong *)new char [(nels*nctypelen(datatype))];
+        // clean up
+        delete[]intg16;
+        delete[]shtbuf;
+    } else if ((datatype == NC_LONG)
+               && (nctypelen(datatype) != sizeof(dods_int32))) {
 
-	if( has_stride)
-	    errstat = lnc_get_vars(ncid, varid, cor, edg, step, lngbuf);
-	else
-	    errstat = lnc_get_vara(ncid, varid, cor, edg, lngbuf);
+        lngbuf = (nclong *) new char[(nels * nctypelen(datatype))];
 
-	if (errstat != NC_NOERR)
-	  throw Error(errstat,string("Could not read the variable `") + name() 
-		      + string("'."));
+        if (has_stride)
+            errstat = lnc_get_vars(ncid, varid, cor, edg, step, lngbuf);
+        else
+            errstat = lnc_get_vara(ncid, varid, cor, edg, lngbuf);
 
-	intg32 = new dods_int32 [nels];
+        if (errstat != NC_NOERR)
+            throw Error(errstat,
+                        string("Could not read the variable `") + name()
+                        + string("'."));
 
-        for (id = 0; id < nels; id++) 
-            *(intg32+id) = (dods_int32) *(lngbuf+id);
+        intg32 = new dods_int32[nels];
 
-	set_read_p(true);  
+        for (id = 0; id < nels; id++)
+            *(intg32 + id) = (dods_int32) * (lngbuf + id);
+
+        set_read_p(true);
         val2buf((void *) intg32);
 
-	// clean up
-        delete [] intg32;
-        delete [] lngbuf;
-    }
-    else if (datatype == NC_CHAR) {
+        // clean up
+        delete[]intg32;
+        delete[]lngbuf;
+    } else if (datatype == NC_CHAR) {
 
-        chrbuf = (char *)new char [(nels*nctypelen(datatype))];
+        chrbuf = (char *) new char[(nels * nctypelen(datatype))];
 
-	// read the vlaues in from the local netCDF file
-	if( has_stride)
-	    errstat = lnc_get_vars_text(ncid, varid, cor, edg, step,chrbuf);
-	else
-	    errstat = lnc_get_vara_text(ncid, varid, cor, edg, chrbuf);
+        // read the vlaues in from the local netCDF file
+        if (has_stride)
+            errstat =
+                lnc_get_vars_text(ncid, varid, cor, edg, step, chrbuf);
+        else
+            errstat = lnc_get_vara_text(ncid, varid, cor, edg, chrbuf);
 
-	if (errstat != NC_NOERR)
-	  throw Error(errstat,string("Could not read the variable `") + name() 
-		      + string("'."));
+        if (errstat != NC_NOERR)
+            throw Error(errstat,
+                        string("Could not read the variable `") + name()
+                        + string("'."));
 
-	string *strg = new string [nels]; // array of strings
-	char buf[2] = " "; // one char and EOS
+        string *strg = new string[nels];        // array of strings
+        char buf[2] = " ";      // one char and EOS
 
-	// put the char values in the string array
-	for (id = 0; id < nels; id++){	  
-	    strncpy(buf, (chrbuf+id), 1);
-	    strg[id] = (string) buf;
-	}
+        // put the char values in the string array
+        for (id = 0; id < nels; id++) {
+            strncpy(buf, (chrbuf + id), 1);
+            strg[id] = (string) buf;
+        }
 
-	// reading is done (dont need to read each individual array value)
-	set_read_p(true);  
-	// put values in the buffers
-	val2buf(strg);
+        // reading is done (dont need to read each individual array value)
+        set_read_p(true);
+        // put values in the buffers
+        val2buf(strg);
 
-	// clean up
-	delete [] strg;
-	delete [] chrbuf;
-    }
-    else {
-      //default (no type conversion needed and the type Byte)
-      char *convbuf = new char [(nels*nctypelen(datatype))];
+        // clean up
+        delete[]strg;
+        delete[]chrbuf;
+    } else {
+        //default (no type conversion needed and the type Byte)
+        char *convbuf = new char[(nels * nctypelen(datatype))];
 
-      if( has_stride)
-	errstat = lnc_get_vars(ncid, varid, cor, edg, step, (void *)convbuf);
-      else
-	errstat = lnc_get_vara(ncid, varid, cor, edg, (void *)convbuf);
- 
-      if (errstat != NC_NOERR)
-	  throw Error(errstat,string("Could not read the variable `") + name() 
-		      + string("'."));
+        if (has_stride)
+            errstat =
+                lnc_get_vars(ncid, varid, cor, edg, step,
+                             (void *) convbuf);
+        else
+            errstat =
+                lnc_get_vara(ncid, varid, cor, edg, (void *) convbuf);
 
-      set_read_p(true);  
-      val2buf((void *)convbuf);
+        if (errstat != NC_NOERR)
+            throw Error(errstat,
+                        string("Could not read the variable `") + name()
+                        + string("'."));
 
-      delete [] convbuf;
+        set_read_p(true);
+        val2buf((void *) convbuf);
+
+        delete[]convbuf;
     }
 
     if (lnc_close(ncid) != NC_NOERR)
-      throw InternalErr(__FILE__, __LINE__, "Could not close the dataset!");
+        throw InternalErr(__FILE__, __LINE__,
+                          "Could not close the dataset!");
 
     return false;
 }
 
-nc_type
-NCArray::get_nc_type() throw(InternalErr)
+nc_type NCArray::get_nc_type() throw(InternalErr)
 {
-    return dynamic_cast<NCAccess*>(var())->get_nc_type();
+    return dynamic_cast < NCAccess * >(var())->get_nc_type();
 }
 
-void
-NCArray::set_source(BaseType *s) throw(InternalErr)
+void NCArray::set_source(BaseType * s) throw(InternalErr)
 {
     if (s->type() == dods_array_c)
-        throw InternalErr(__FILE__, __LINE__, "Array's source is an Array!");
-     
+        throw InternalErr(__FILE__, __LINE__,
+                          "Array's source is an Array!");
+
     d_source = s->ptr_duplicate();
     DBG(cerr << "s " << s << ", d_source " << d_source << endl);
 }
@@ -652,38 +631,40 @@ NCArray::set_source(BaseType *s) throw(InternalErr)
     @todo Optimize by not doing all that if the contained variable is
     an atomic type. */
 VarList
-NCArray::flatten(const ClientParams &cp, const string &parent_name)
+    NCArray::flatten(const ClientParams & cp, const string & parent_name)
 {
 #if 0
     BaseType *btp = var();
-    NCAccess *nca = dynamic_cast<NCAccess*>(btp);
+    NCAccess *nca = dynamic_cast < NCAccess * >(btp);
     VarList template_vars = nca->flatten(cp, parent_name);
 #else
-    VarList template_vars = dynamic_cast<NCAccess*>(var())->flatten(cp, parent_name);
+    VarList template_vars =
+        dynamic_cast < NCAccess * >(var())->flatten(cp, parent_name);
 #endif
     // At this point we can delete the original template variable (which 
     // might be a Structure or Sequence, so there may be several variables
     // in place of the single template). Deleting the template keeps from
     // copying it repeatedly in the while-loop.
     add_var(0);
-    
+
     VarListIter tv = template_vars.begin();
     VarListIter tv_end = template_vars.end();
     VarList new_vars;
     while (tv != tv_end) {
         // Copy this, cast to Array, load the template, push to list
-        Array *ap = dynamic_cast<Array*>(ptr_duplicate());
+        NCArray *ap = dynamic_cast < NCArray * >(ptr_duplicate());
+        ap->set_translated(true);
         ap->add_var(*tv);
         ap->set_name((*tv)->name());
         new_vars.push_back(ap);
-        delete *tv; *tv++ = 0;
+        delete *tv;
+        *tv++ = 0;
     }
-    
+
     return new_vars;
 }
 
-BaseType *
-NCArray::find_child_sequence()
+BaseType *NCArray::find_child_sequence()
 {
     if (var()->type() == dods_sequence_c)
         return var();
