@@ -78,8 +78,8 @@ public:
 
     virtual nc_type get_nc_type() throw(InternalErr);
     
-    virtual void extract_values(void *values, int elements, int outtype)
-        throw(Error);
+    virtual void extract_values(void *values, int elements, int outtype,
+                                const ClientParams &/* cp */) throw(Error);
     // virtual BaseType *get_source();
     virtual void set_source(BaseType *s) throw(InternalErr);
     
