@@ -38,7 +38,7 @@
 
 #include "config_nc.h"
 
-static char rcsid[] not_used ={"$Id: NCGrid.cc 11906 2005-08-08 19:51:43Z root $"};
+static char rcsid[] not_used ={"$Id: NCGrid.cc 13531 2006-03-28 21:40:06Z jimg $"};
 
 #include <sstream>
 
@@ -206,9 +206,11 @@ NCGrid::is_convertable(int outtype)
 }
 
 void
-NCGrid::extract_values(void *values, int elements, int outtype) throw(Error)
+NCGrid::extract_values(void *values, int elements, int outtype,
+                       const ClientParams &cp) throw(Error)
 {
-    dynamic_cast<NCAccess*>(array_var())->extract_values(values, elements, outtype);
+    dynamic_cast<NCAccess*>(array_var())->extract_values(values, elements,
+                                                         outtype, cp);
 }
 
 nc_type
