@@ -64,8 +64,8 @@ public:
     virtual bool is_convertable(int outtype);
     virtual nc_type get_nc_type() throw(InternalErr);
     
-    virtual void extract_values(void *values, int elements, int outtype)
-        throw(Error);
+    virtual void extract_values(void *values, int elements, int outtype,
+                                const ClientParams &cp) throw(Error);
 };
 
 /* 
