@@ -56,8 +56,8 @@ public:
 
     virtual BaseType *ptr_duplicate();
     virtual nc_type get_nc_type() throw(InternalErr);
-    virtual void extract_values(void *values, int elements, int outtype)
-        throw(Error);
+    virtual void extract_values(void *values, int elements, int outtype,
+                                const ClientParams &cp) throw(Error);
     
     virtual bool read(const string &dataset);
 };
