@@ -38,7 +38,7 @@
 
 #include "config_nc.h"
 
-static char rcsid[] not_used ={"$Id: NCUrl.cc 11992 2005-08-22 19:40:47Z jimg $"};
+static char rcsid[] not_used ={"$Id: NCUrl.cc 13725 2006-04-19 18:06:01Z jimg $"};
 
 
 #include "InternalErr.h"
@@ -85,7 +85,7 @@ NCUrl::ptr_duplicate()
 }
 
 void
-NCUrl::extract_values(void *values, int outtype) throw(Error)
+NCUrl::extract_values(void *values, int /*outtype*/) throw(Error)
 {
     char * tbfr = (char *)values;
     string *cp = 0;
