@@ -3,7 +3,7 @@
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
 
-/* $Id: fort-v2compat.c 11906 2005-08-08 19:51:43Z root $ */
+/* $Id: fort-v2compat.c 15015 2006-11-03 17:39:01Z jimg $ */
 
 /*
  *  Source for netCDF FORTRAN jacket library.
@@ -22,9 +22,13 @@
  * differing error handling mechanisms between version 2 and version 3.
  */
 
+#include <config.h>
+
+#ifndef NO_NETCDF_2
+
 /* LINTLIBRARY */
 
-#include	"ncconfig.h"
+#include	<config.h>
 #include	<ctype.h>
 #include        <string.h>
 #include	<stdlib.h>
@@ -66,13 +70,11 @@
 static ptrdiff_t*
 f2c_v2imap(int ncid, int varid, const int* fimap, ptrdiff_t* cimap)
 {
-    int		status;
     int		rank;
     nc_type	datatype;
 
-    if ((status = nc_inq_vartype(ncid, varid, &datatype)) != NC_NOERR ||
-	(status = nc_inq_varndims(ncid, varid, &rank)) != NC_NOERR ||
-	rank <= 0)
+    if (nc_inq_vartype(ncid, varid, &datatype) ||
+	nc_inq_varndims(ncid, varid, &rank) || rank <= 0)
     {
 	return NULL;
     }
@@ -187,20 +189,6 @@ dimprod(const size_t* count, int rank)
 }
 
 
-/*
- * Divide a mapping vector specified in units of bytes by the size of the
- * relevant element.
- */
-static void
-maps_divide(const ptrdiff_t* basis, int rank, size_t size, ptrdiff_t* maps)
-{
-    int	i;
-
-    for (i = 0; i < rank; ++i)
-	maps[i] = basis[i] / size;
-}
-
-
 /*
  * Set the C global variable ncopts.
  */
@@ -239,7 +227,7 @@ FCALLSCSUB1(c_ncgopt,NCGOPT,ncgopt,
  */
 static int
 c_nccre(
-    char *pathname,	/* file name of new netCDF file */
+    const char *pathname,	/* file name of new netCDF file */
     int clobmode,	/* either NCCLOB or NCNOCLOB */
     int *rcode		/* returned error code */
 )
@@ -287,7 +275,7 @@ FCALLSCFUN3(NF_INT,c_nccre,NCCRE,nccre,
  */
 static int
 c_ncopn(
-    char *pathname,		/* file name for netCDF to be opened */
+    const char *pathname,	/* file name for netCDF to be opened */
     int rwmode,			/* either NCWRITE or NCNOWRIT */
     int *rcode			/* returned error code */
 )
@@ -337,7 +325,7 @@ FCALLSCFUN3(NF_INT,c_ncopn,NCOPN,ncopn,
 static int
 c_ncddef (
     int ncid,		/* netCDF ID */
-    char *dimname,	/* dimension name */
+    const char *dimname,/* dimension name */
     int dimlen,		/* size of dimension */
     int *rcode		/* returned error code */
 )
@@ -366,7 +354,7 @@ FCALLSCFUN4(NF_INT,c_ncddef,NCDDEF,ncddef,
 static int
 c_ncdid (
     int ncid,		/* netCDF ID */
-    char *dimname,	/* dimension name */
+    const char *dimname,/* dimension name */
     int *rcode		/* returned error code */
 )
 {
@@ -394,7 +382,7 @@ FCALLSCFUN3(NF_INT,c_ncdid,NCDID,ncdid,
 static int
 c_ncvdef (
     int ncid,		/* netCDF ID */
-    char *varname,	/* name of variable */
+    const char *varname,/* name of variable */
     nc_type datatype,	/* netCDF datatype of variable */
     int ndims,		/* number of dimensions of variable */
     int *dimids,	/* array of ndims dimensions IDs */
@@ -429,7 +417,7 @@ FCALLSCFUN6(NF_INT,c_ncvdef,NCVDEF,ncvdef,
 static int
 c_ncvid (
     int ncid,		/* netCDF ID */
-    char *varname,	/* variable name */
+    const char *varname,/* variable name */
     int *rcode		/* returned error code */
 )
 {
@@ -630,7 +618,7 @@ static void
 c_ncdren (
     int ncid,			/* netCDF ID */
     int dimid,			/* dimension ID */
-    char* dimname,		/* new name of dimension */
+    const char* dimname,	/* new name of dimension */
     int* rcode			/* returned error code */
 )
 {
@@ -1562,7 +1550,7 @@ static void
 c_ncvren (
     int ncid,		/* netCDF ID */
     int varid,		/* variable ID */
-    char* varname,	/* new name for variable */
+    const char* varname,/* new name for variable */
     int* rcode		/* returned error code */
 )
 {
@@ -1883,7 +1871,7 @@ static void
 c_ncacpy (
     int inncid,		/* input netCDF ID */
     int invarid,	/* variable ID of input netCDF or NC_GLOBAL */
-    char* attname,	/* name of attribute in input netCDF to be copied */
+    const char* attname,/* name of attribute in input netCDF to be copied */
     int outncid,	/* ID of output netCDF file for attribute */
     int outvarid,	/* ID of associated netCDF variable or NC_GLOBAL */
     int* rcode		/* returned error code */
@@ -1930,8 +1918,8 @@ static void
 c_ncaren (
     int ncid,		/* netCDF ID */
     int varid,		/* variable ID */
-    char* attname,	/* attribute name */
-    char* newname,	/* new name */
+    const char* attname,/* attribute name */
+    const char* newname,/* new name */
     int* rcode		/* returned error code */
 )
 {
@@ -1952,7 +1940,7 @@ static void
 c_ncadel (
     int ncid,		/* netCDF ID */
     int varid,		/* variable ID */
-    char* attname,	/* attribute name */
+    const char* attname,/* attribute name */
     int* rcode		/* returned error code */
 )
 {
@@ -1988,3 +1976,5 @@ c_ncsfil (
 /* FORTRAN interface to the above. */
 FCALLSCFUN3(NF_INT,c_ncsfil,NCSFIL,ncsfil,
     NCID,FILLMODE,PRCODE)
+
+#endif /*!NO_NETCDF_2*/
