@@ -96,6 +96,12 @@ lnc_inq_natts(int ncid, int *nattsp);
 EXTERNL int 
 lnc_inq_unlimdim(int ncid, int *unlimdimidp);
 
+EXTERNL int
+lnc_set_default_format(int format, int *old_formatp);
+
+EXTERNL int
+lnc_inq_format(int ncid, int *formatp);
+
 /* Begin _dim */
 
 EXTERNL int
