@@ -2,15 +2,21 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: lnc.h 11906 2005-08-08 19:51:43Z root $ */
+/* $Id: lnc.h 14988 2006-11-01 22:21:09Z jimg $ */
 #ifndef _NC_H_
 #define _NC_H_
 
 /*
  *	netcdf library 'private' data structures, objects and interfaces
  */
+#include <config.h>
+
+/* If netcdf-4 is in use, rename all nc_ functions to nc3_ functions. */
+#ifdef USE_NETCDF4
+#include <netcdf3.h>
+#include <nc3convert.h>
+#endif
 
-#include	"ncconfig.h"
 #include	<stddef.h>	/* size_t */
 #include	<sys/types.h>	/* off_t */
 #include	"lnetcdf.h"
@@ -216,6 +222,9 @@ NC_var_shape(NC_var *varp, const NC_dimarray *dims);
 extern int
 NC_findvar(const NC_vararray *ncap, const char *name, NC_var **varpp);
 
+extern int
+NC_check_vlen(NC_var *varp, size_t vlen_max);
+
 extern NC_var *
 NC_lookupvar(NC *ncp, int varid);
 
@@ -240,8 +249,8 @@ struct NC {
 	/* contains the previous NC during redef. */
 	struct NC *old;
 	/* flags */
-#define NC_CREAT 2	/* in create phase, cleared by lncendef */
-#define NC_INDEF 8	/* in define mode, cleared by lncendef */
+#define NC_CREAT 2	/* in create phase, cleared by ncendef */
+#define NC_INDEF 8	/* in define mode, cleared by ncendef */
 #define NC_NSYNC 0x10	/* synchronise numrecs on change */
 #define NC_HSYNC 0x20	/* synchronise whole header on change */
 #define NC_NDIRTY 0x40	/* numrecs has changed */
@@ -251,9 +260,14 @@ struct NC {
 	ncio *nciop;
 	size_t chunk;	/* largest extent this layer will request from ncio->get() */
 	size_t xsz;	/* external size of this header, == var[0].begin */
-	off_t begin_var; /* postion of the first (non-record) var */
-	off_t begin_rec; /* postion of the first 'record' */
-	size_t recsize; /* length of 'record' */
+	off_t begin_var; /* position of the first (non-record) var */
+	off_t begin_rec; /* position of the first 'record' */
+        /* don't constrain maximum size of record unnecessarily */
+#if SIZEOF_OFF_T > SIZEOF_SIZE_T
+        off_t recsize;   /* length of 'record' */
+#else
+	size_t recsize;  /* length of 'record' */
+#endif
 	/* below gets xdr'd */
 	size_t numrecs; /* number of 'records' allocated */
 	NC_dimarray dims;
@@ -327,7 +341,7 @@ extern int
 nc_cktype(nc_type datatype);
 
 extern size_t
-lncx_howmany(nc_type type, size_t xbufsize);
+ncx_howmany(nc_type type, size_t xbufsize);
 
 extern int
 read_numrecs(NC *ncp);
@@ -338,11 +352,14 @@ write_numrecs(NC *ncp);
 extern int
 NC_sync(NC *ncp);
 
+extern int
+NC_calcsize(NC *ncp, off_t *filesizep);
+
 /* End defined in nc.c */
 /* Begin defined in v1hpg.c */
 
 extern size_t
-ncx_len_NC(const NC *ncp);
+ncx_len_NC(const NC *ncp, size_t sizeof_off_t);
 
 extern int
 ncx_put_NC(const NC *ncp, void **xpp, off_t offset, size_t extent);
@@ -354,7 +371,7 @@ lnc_get_NC( NC *ncp);
 /* Begin defined in putget.c */
 
 extern int
-fill_NC_var(NC *ncp, const NC_var *varp, size_t recno);
+fill_NC_var(NC *ncp, const NC_var *varp, size_t varsize, size_t recno);
 
 extern int
 lnc_inq_rec(int ncid, size_t *nrecvars, int *recvarids, size_t *recsizes);
@@ -367,53 +384,4 @@ lnc_put_rec(int ncid, size_t recnum, void *const *datap);
 
 /* End defined in putget.c */
 
-/*
- * These functions are used to support
- * interface version 2 backward compatiblity.
- * N.B. these are tested in ../nc_test even though they are
- * not public. So, be careful to change the declarations in
- * ../nc_test/tests.h if you change these.
- */
-
-extern int
-lnc_put_att(int ncid, int varid, const char *name, nc_type datatype,
-	size_t len, const void *value);
-
-extern int
-lnc_get_att(int ncid, int varid, const char *name, void *value);
-
-extern int
-lnc_put_var1(int ncid, int varid, const size_t *index, const void *value);
-
-extern int
-lnc_get_var1(int ncid, int varid, const size_t *index, void *value);
-
-extern int
-lnc_put_vara(int ncid, int varid,
-	 const size_t *start, const size_t *count, const void *value);
-
-extern int
-lnc_get_vara(int ncid, int varid,
-	 const size_t *start, const size_t *count, void *value);
-
-extern int
-lnc_put_vars(int ncid, int varid,
-	 const size_t *start, const size_t *count, const ptrdiff_t *stride,
-	 const void * value);
-
-extern int
-lnc_get_vars(int ncid, int varid,
-	 const size_t *start, const size_t *count, const ptrdiff_t *stride,
-	 void * value);
-
-extern int
-lnc_put_varm(int ncid, int varid,
-	 const size_t *start, const size_t *count, const ptrdiff_t *stride,
-	 const ptrdiff_t * map, const void *value);
-
-extern int
-lnc_get_varm(int ncid, int varid,
-	 const size_t *start, const size_t *count, const ptrdiff_t *stride,
-	 const ptrdiff_t * map, void *value);
-
 #endif /* _NC_H_ */
