@@ -1,5 +1,5 @@
 /*
- * Copyright 1993-1996 University Corporation for Atmospheric Research/Unidata
+ * Copyright 1993-2005 University Corporation for Atmospheric Research/Unidata
  * 
  * Portions of this software were developed by the Unidata Program at the 
  * University Corporation for Atmospheric Research.
@@ -30,8 +30,8 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-/* "$Id: lnetcdf.h 11906 2005-08-08 19:51:43Z root $" */
-
+/* "$Id: lnetcdf.h 14988 2006-11-01 22:21:09Z jimg $" */
+
 #ifndef _NETCDF_
 #define _NETCDF_
 
@@ -78,20 +78,43 @@ typedef enum {
  * will be used as the fill value for that variable.
  */
 #define _FillValue	"_FillValue"
-
+#define NC_FILL		0	/* argument to ncsetfill to clear NC_NOFILL */
+#define NC_NOFILL	0x100	/* Don't fill data section an records */
 
 /*
- * 'mode' flags for lnccreate and lncopen
+ * 'mode' flags for ncopen
  */
 #define NC_NOWRITE	0	/* default is read only */
 #define NC_WRITE    	0x1	/* read & write */
+
+/*
+ * 'mode' flags for nccreate
+ */
 #define NC_CLOBBER	0
 #define NC_NOCLOBBER	0x4	/* Don't destroy existing file on create */
-#define NC_FILL		0	/* argument to lncsetfill to clear NC_NOFILL */
-#define NC_NOFILL	0x100	/* Don't fill data section an records */
-#define NC_LOCK		0x0400	/* Use locking if available */
+#define NC_64BIT_OFFSET 0x0200  /* Use large (64-bit) file offsets */
+
+/*
+ * 'mode' flags for nccreate and ncopen
+ */
 #define NC_SHARE	0x0800	/* Share updates, limit cacheing */
-
+#define NC_STRICT_NC3  (0x8)
+
+/* The following flag currently is ignored, but use in
+ * lnc_open() or lnc_create() may someday support use of advisory
+ * locking to prevent multiple writers from clobbering a file 
+ */
+#define NC_LOCK		0x0400	/* Use locking if available */
+
+/*
+ * Starting with version 3.6, there are different format netCDF
+ * files. 4.0 instroduces the third one.
+ */
+#define NC_FORMAT_CLASSIC (1)
+#define NC_FORMAT_64BIT   (2)
+#define NC_FORMAT_NETCDF4 (3)
+#define NC_FORMAT_NETCDF4_CLASSIC  (4) /* create netcdf-4 files, with NC_STRICT_NC3. */
+
 /*
  * Let lnc__create() or lnc__open() figure out
  * as suitable chunk size.
@@ -104,7 +127,7 @@ typedef enum {
 #define NC_ALIGN_CHUNK ((size_t)(-1))
 
 /*
- * 'size' argument to lncdimdef for an unlimited dimension
+ * 'size' argument to ncdimdef for an unlimited dimension
  */
 #define NC_UNLIMITED 0L
 
@@ -113,40 +136,43 @@ typedef enum {
  */
 #define NC_GLOBAL -1
 
+/* These are in support of the coordinate axis stuff. */
+#define NC_NOAXISTYPE 0
+#define NC_LATITUDE 1
+#define NC_LONGITUDE 2
+#define NC_GEOX 3
+#define NC_GEOY 4
+#define NC_GEOZ 5
+#define NC_HEIGHT_UP 6
+#define NC_HEIGHT_DOWN 7
+#define NC_PRESSURE 8
+#define NC_TIME 9
+#define NC_RADAZ 10
+#define NC_RADEL 11
+#define NC_RADDIST 12
 
 /*
  * These maximums are enforced by the interface, to facilitate writing
  * applications and utilities.  However, nothing is statically allocated to
  * these sizes internally.
  */
-#if 0
-#define NC_MAX_DIMS	100	 /* max dimensions per file */
-#define NC_MAX_ATTRS	2000	 /* max global or per variable attributes */
-#define NC_MAX_VARS	2000	 /* max variables per file */
-#define NC_MAX_NAME	128	 /* max length of a name */
+#define NC_MAX_DIMS	1024	 /* max dimensions per file */
+#define NC_MAX_ATTRS	8192	 /* max global or per variable attributes */
+#define NC_MAX_VARS	8192	 /* max variables per file */
+#define NC_MAX_NAME	256	 /* max length of a name */
 #define NC_MAX_VAR_DIMS	NC_MAX_DIMS /* max per variable dimensions */
-#else
-#define NC_MAX_DIMS     1024     /* max dimensions per file */
-#define NC_MAX_ATTRS    4096     /* max global or per variable attributes */
-#define NC_MAX_VARS     4096     /* max variables per file */
-#define NC_MAX_NAME     1024      /* max length of a name */
-#define NC_MAX_VAR_DIMS NC_MAX_DIMS /* max per variable dimensions */
-#endif
-
+
 /*
  * The netcdf version 3 functions all return integer error status.
  * These are the possible values, in addition to certain
  * values from the system errno.h.
- * 
- * I've added error codes above 1000 as opendap error codes. These
- * should all fall below the codes in errno.h; I'm using 1000++ becuase
- * those are the values used by the DAP library. jhrg 3/2/05
  */
 
-#define NC_ISSYSERR(err)        (((err) > 0) && ((err) < 1000))
+#define NC_ISSYSERR(err)	((err) > 0)
 
 #define	NC_NOERR	0	/* No Error */
 
+#define NC2_ERR         (-1)    /* Returned for all errors in the v2 API. */
 #define	NC_EBADID	(-33)	/* Not a netcdf id */
 #define	NC_ENFILE	(-34)	/* Too many netcdfs open */
 #define	NC_EEXIST	(-35)	/* netcdf file exists && NC_NOCLOBBER */
@@ -171,7 +197,7 @@ typedef enum {
 #define NC_EUNLIMIT    	(-54)	/* NC_UNLIMITED size already in use */
 #define NC_ENORECVARS  	(-55)	/* nc_rec op when there are no record vars */
 #define NC_ECHAR	(-56)	/* Attempt to convert between text & numbers */
-#define NC_EEDGE	(-57)	/* Edge+start exceeds dimension bound */
+#define NC_EEDGE	(-57)	/* Start+count exceeds dimension bound */
 #define NC_ESTRIDE	(-58)	/* Illegal stride */
 #define NC_EBADNAME	(-59)	/* Attribute or variable name
                                          contains illegal characters */
@@ -179,6 +205,11 @@ typedef enum {
 #define NC_ERANGE	(-60)	/* Math result not representable */
 #define NC_ENOMEM	(-61)	/* Memory allocation (malloc) failure */
 
+#define NC_EVARSIZE     (-62)   /* One or more variable sizes violate
+				   format constraints */ 
+#define NC_EDIMSIZE     (-63)   /* Invalid dimension size */
+#define NC_ETRUNC       (-64)   /* File likely truncated or possibly corrupted */
+
 /* These are specific to the OPeNDAP netCDF 3.x client library. */
 #define OP_EUNDEF 1000 /** Undefined error code (undefined_error) */
 #define OP_EUNKNOWN 1001 /** Unknown error (unknown_error) */
@@ -190,7 +221,6 @@ typedef enum {
 #define OP_EREADFAIL 1007 /** (can_not_read_file) */
 #define OP_EREAD_FAIL 1008 /** (cannot_read_file) */
 
-
 /*
  * The Interface
  */
@@ -203,12 +233,19 @@ typedef enum {
 #  else
 #   define MSC_EXTRA __declspec(dllimport)
 #  endif
+#include <io.h>
+#define lseek _lseeki64
+#define off_t __int64
 #else
 #define MSC_EXTRA
 #endif	/* defined(DLL_NETCDF) */
 
 # define EXTERNL extern MSC_EXTRA
 
+#if defined(DLL_NETCDF) /* define when library is a DLL */
+MSC_EXTRA int ncerr;
+MSC_EXTRA int ncopts;
+#endif
 EXTERNL const char *
 lnc_inq_libvers(void);
 
@@ -265,7 +302,13 @@ lnc_inq_natts(int ncid, int *nattsp);
 
 EXTERNL int 
 lnc_inq_unlimdim(int ncid, int *unlimdimidp);
-
+
+EXTERNL int
+nc_set_default_format(int format, int *old_formatp);
+
+EXTERNL int
+lnc_inq_format(int ncid, int *formatp);
+
 /* Begin _dim */
 
 EXTERNL int
@@ -291,7 +334,7 @@ lnc_rename_dim(int ncid, int dimid, const char *name);
 
 EXTERNL int
 lnc_inq_att(int ncid, int varid, const char *name,
-	 nc_type *xtypep, size_t *lenp);
+	   nc_type *xtypep, size_t *lenp);
 
 EXTERNL int 
 lnc_inq_attid(int ncid, int varid, const char *name, int *idp);
@@ -314,12 +357,19 @@ lnc_rename_att(int ncid, int varid, const char *name, const char *newname);
 EXTERNL int
 lnc_del_att(int ncid, int varid, const char *name);
 
-/* End _att */
+/* End _att */
 /* Begin {put,get}_att */
 
+EXTERNL int
+lnc_put_att(int ncid, int varid, const char *name, nc_type datatype,
+	   size_t len, const void *value);
+
+EXTERNL int
+lnc_get_att(int ncid, int varid, const char *name, void *value);
+
 EXTERNL int
 lnc_put_att_text(int ncid, int varid, const char *name,
-	size_t len, const char *op);
+		size_t len, const char *op);
 
 EXTERNL int
 lnc_get_att_text(int ncid, int varid, const char *name, char *ip);
@@ -373,16 +423,16 @@ lnc_put_att_double(int ncid, int varid, const char *name, nc_type xtype,
 EXTERNL int
 lnc_get_att_double(int ncid, int varid, const char *name, double *ip);
 
-/* End {put,get}_att */
+/* End {put,get}_att */
 /* Begin _var */
 
 EXTERNL int
-lnc_def_var(int ncid, const char *name,
-	 nc_type xtype, int ndims, const int *dimidsp, int *varidp);
+lnc_def_var(int ncid, const char *name, nc_type xtype, int ndims, 
+	   const int *dimidsp, int *varidp);
 
 EXTERNL int
-lnc_inq_var(int ncid, int varid, char *name,
-	 nc_type *xtypep, int *ndimsp, int *dimidsp, int *nattsp);
+lnc_inq_var(int ncid, int varid, char *name, nc_type *xtypep, 
+	   int *ndimsp, int *dimidsp, int *nattsp);
 
 EXTERNL int
 lnc_inq_varid(int ncid, const char *name, int *varidp);
@@ -412,9 +462,15 @@ lnc_copy_var(int ncid_in, int varid, int ncid_out);
 #define ncvarcpy(ncid_in, varid, ncid_out) ncvarcopy((ncid_in), (varid), (ncid_out))
 #endif
 
-/* End _var */
+/* End _var */
 /* Begin {put,get}_var1 */
 
+EXTERNL int
+lnc_put_var1(int ncid, int varid, const size_t *indexp, const void *value);
+
+EXTERNL int
+lnc_get_var1(int ncid, int varid, const size_t *indexp, void *value);
+
 EXTERNL int
 lnc_put_var1_text(int ncid, int varid, const size_t *indexp, const char *op);
 
@@ -472,6 +528,14 @@ lnc_get_var1_double(int ncid, int varid, const size_t *indexp, double *ip);
 /* End {put,get}_var1 */
 /* Begin {put,get}_vara */
 
+EXTERNL int
+lnc_put_vara(int ncid, int varid,
+	 const size_t *start, const size_t *count, const void *value);
+
+EXTERNL int
+lnc_get_vara(int ncid, int varid,
+	 const size_t *start, const size_t *count, void *value);
+
 EXTERNL int
 lnc_put_vara_text(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const char *op);
@@ -479,7 +543,7 @@ lnc_put_vara_text(int ncid, int varid,
 EXTERNL int
 lnc_get_vara_text(int ncid, int varid,
 	const size_t *startp, const size_t *countp, char *ip);
-
+
 EXTERNL int
 lnc_put_vara_uchar(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const unsigned char *op);
@@ -536,9 +600,19 @@ EXTERNL int
 lnc_get_vara_double(int ncid, int varid,
 	const size_t *startp, const size_t *countp, double *ip);
 
-/* End {put,get}_vara */
+/* End {put,get}_vara */
 /* Begin {put,get}_vars */
 
+EXTERNL int
+lnc_put_vars(int ncid, int varid,
+	 const size_t *start, const size_t *count, const ptrdiff_t *stride,
+	 const void * value);
+
+EXTERNL int
+lnc_get_vars(int ncid, int varid,
+	 const size_t *start, const size_t *count, const ptrdiff_t *stride,
+	 void * value);
+
 EXTERNL int
 lnc_put_vars_text(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
@@ -598,7 +672,7 @@ EXTERNL int
 lnc_get_vars_long(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
 	long *ip);
-
+
 EXTERNL int
 lnc_put_vars_float(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
@@ -622,6 +696,15 @@ lnc_get_vars_double(int ncid, int varid,
 /* End {put,get}_vars */
 /* Begin {put,get}_varm */
 
+EXTERNL int
+lnc_put_varm(int ncid, int varid, const size_t *start, const size_t *count, 
+            const ptrdiff_t *stride, const ptrdiff_t *imapp, 
+	    const void *value);
+
+EXTERNL int
+lnc_get_varm(int ncid, int varid, const size_t *start, const size_t *count, 
+	    const ptrdiff_t *stride, const ptrdiff_t *imapp, void *value);
+
 EXTERNL int
 lnc_put_varm_text(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
@@ -657,7 +740,7 @@ lnc_get_varm_schar(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
 	const ptrdiff_t *imapp, 
 	signed char *ip);
-
+
 EXTERNL int
 lnc_put_varm_short(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
@@ -715,7 +798,7 @@ lnc_put_varm_double(int ncid, int varid,
 EXTERNL int
 lnc_get_varm_double(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
-	const ptrdiff_t * imap, 
+	const ptrdiff_t * imapp, 
 	double *ip);
 
 /* End {put,get}_varm */
@@ -769,6 +852,25 @@ lnc_put_var_double(int ncid, int varid, const double *op);
 EXTERNL int
 lnc_get_var_double(int ncid, int varid, double *ip);
 
+#ifdef LOGGING
+
+#ifdef DEBUG
+EXTERNL void
+nc_exit(void);
+#endif
+
+EXTERNL void 
+nc_set_log_level(int new_level);
+/* Use this to turn off logging by calling
+   nc_log_level(NC_TURN_OFF_LOGGING) */
+#define NC_TURN_OFF_LOGGING (-1)
+
+#else /* not LOGGING */
+
+#define nc_set_log_level(e)
+
+#endif
+
 /* End {put,get}_var */
 
 /* #ifdef _CRAYMPP */
@@ -794,18 +896,20 @@ EXTERNL int
 lnc__open_mp(const char *path, int mode, int basepe,
 	size_t *chunksizehintp, int *ncidp);
 
+EXTERNL int
+lnc_delete(const char * path);
+
 EXTERNL int
 lnc_delete_mp(const char * path, int basepe);
 
 EXTERNL int
-lnc_set_base_pe(int ncid, int pe);
+nc_set_base_pe(int ncid, int pe);
 
 EXTERNL int
 lnc_inq_base_pe(int ncid, int *pe);
 
 /* #endif _CRAYMPP */
 
-
 /* Begin v2.4 backward compatiblity */
 /*
  * defining NO_NETCDF_2 to the preprocessor
@@ -872,124 +976,124 @@ lnc_advise(const char *cdf_routine_name, int err, const char *fmt,...);
  * This is the only thing in this file which architecture dependent.
  */
 typedef int nclong;
-
-EXTERNL int
-lnctypelen(nc_type datatype);
 
 EXTERNL int
-lnccreate(const char* path, int cmode);
+nctypelen(nc_type datatype);
 
 EXTERNL int
-lncopen(const char* path, int mode);
+nccreate(const char* path, int cmode);
 
 EXTERNL int
-lncsetfill(int ncid, int fillmode);
+ncopen(const char* path, int mode);
 
 EXTERNL int
-lncredef(int ncid);
+ncsetfill(int ncid, int fillmode);
 
 EXTERNL int
-lncendef(int ncid);
+ncredef(int ncid);
 
 EXTERNL int
-lncsync(int ncid);
+ncendef(int ncid);
 
 EXTERNL int
-lncabort(int ncid);
+ncsync(int ncid);
 
 EXTERNL int
-lncclose(int ncid);
+ncabort(int ncid);
 
 EXTERNL int
-lncinquire(int ncid, int *ndimsp, int *nvarsp, int *nattsp, int *unlimdimp);
+ncclose(int ncid);
 
 EXTERNL int
-lncdimdef(int ncid, const char *name, long len);
+ncinquire(int ncid, int *ndimsp, int *nvarsp, int *nattsp, int *unlimdimp);
 
 EXTERNL int
-lncdimid(int ncid, const char *name);
+ncdimdef(int ncid, const char *name, long len);
 
 EXTERNL int
-lncdiminq(int ncid, int dimid, char *name, long *lenp);
+ncdimid(int ncid, const char *name);
 
 EXTERNL int
-lncdimrename(int ncid, int dimid, const char *name);
-
+ncdiminq(int ncid, int dimid, char *name, long *lenp);
+
 EXTERNL int
-lncattput(int ncid, int varid, const char *name, nc_type xtype,
+ncdimrename(int ncid, int dimid, const char *name);
+
+EXTERNL int
+ncattput(int ncid, int varid, const char *name, nc_type xtype,
 	int len, const void *op);
 
 EXTERNL int
-lncattinq(int ncid, int varid, const char *name, nc_type *xtypep, int *lenp);
+ncattinq(int ncid, int varid, const char *name, nc_type *xtypep, int *lenp);
 
 EXTERNL int
-lncattget(int ncid, int varid, const char *name, void *ip);
+ncattget(int ncid, int varid, const char *name, void *ip);
 
 EXTERNL int
-lncattcopy(int ncid_in, int varid_in, const char *name, int ncid_out,
+ncattcopy(int ncid_in, int varid_in, const char *name, int ncid_out,
 	int varid_out);
 
 EXTERNL int
-lncattname(int ncid, int varid, int attnum, char *name);
+ncattname(int ncid, int varid, int attnum, char *name);
 
 EXTERNL int
-lncattrename(int ncid, int varid, const char *name, const char *newname);
+ncattrename(int ncid, int varid, const char *name, const char *newname);
 
 EXTERNL int
-lncattdel(int ncid, int varid, const char *name);
+ncattdel(int ncid, int varid, const char *name);
 
 EXTERNL int
-lncvardef(int ncid, const char *name, nc_type xtype,
+ncvardef(int ncid, const char *name, nc_type xtype,
 	int ndims, const int *dimidsp);
 
 EXTERNL int
-lncvarid(int ncid, const char *name);
+ncvarid(int ncid, const char *name);
 
 EXTERNL int
-lncvarinq(int ncid, int varid, char *name, nc_type *xtypep,
+ncvarinq(int ncid, int varid, char *name, nc_type *xtypep,
 	int *ndimsp, int *dimidsp, int *nattsp);
 
 EXTERNL int
-lncvarput1(int ncid, int varid, const long *indexp, const void *op);
+ncvarput1(int ncid, int varid, const long *indexp, const void *op);
 
 EXTERNL int
-lncvarget1(int ncid, int varid, const long *indexp, void *ip);
+ncvarget1(int ncid, int varid, const long *indexp, void *ip);
 
 EXTERNL int
-lncvarput(int ncid, int varid, const long *startp, const long *countp,
+ncvarput(int ncid, int varid, const long *startp, const long *countp,
 	const void *op);
 
 EXTERNL int
-lncvarget(int ncid, int varid, const long *startp, const long *countp, 
+ncvarget(int ncid, int varid, const long *startp, const long *countp, 
 	void *ip);
-
+
 EXTERNL int
-lncvarputs(int ncid, int varid, const long *startp, const long *countp,
+ncvarputs(int ncid, int varid, const long *startp, const long *countp,
 	const long *stridep, const void *op);
 
 EXTERNL int
-lncvargets(int ncid, int varid, const long *startp, const long *countp,
+ncvargets(int ncid, int varid, const long *startp, const long *countp,
 	const long *stridep, void *ip);
 
 EXTERNL int
-lncvarputg(int ncid, int varid, const long *startp, const long *countp,
+ncvarputg(int ncid, int varid, const long *startp, const long *countp,
 	const long *stridep, const long *imapp, const void *op);
 
 EXTERNL int
-lncvargetg(int ncid, int varid, const long *startp, const long *countp,
+ncvargetg(int ncid, int varid, const long *startp, const long *countp,
 	const long *stridep, const long *imapp, void *ip);
 
 EXTERNL int
-lncvarrename(int ncid, int varid, const char *name);
+ncvarrename(int ncid, int varid, const char *name);
 
 EXTERNL int
-lncrecinq(int ncid, int *nrecvarsp, int *recvaridsp, long *recsizesp);
+ncrecinq(int ncid, int *nrecvarsp, int *recvaridsp, long *recsizesp);
 
 EXTERNL int
-lncrecget(int ncid, long recnum, void **datap);
+ncrecget(int ncid, long recnum, void **datap);
 
 EXTERNL int
-lncrecput(int ncid, long recnum, void *const *datap);
+ncrecput(int ncid, long recnum, void *const *datap);
 
 /* End v2.4 backward compatiblity */
 #endif /*!NO_NETCDF_2*/
