@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1996, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Id: error.c 11906 2005-08-08 19:51:43Z root $
+ *   $Id: error.c 15015 2006-11-03 17:39:01Z jimg $
  *********************************************************************/
 
 #include <stddef.h>	/* because gcc 2.7.2.2 doesn't define size_t */
@@ -9,8 +9,8 @@
 #include <stdio.h>
 #include <stdarg.h>
 
-extern int  nfails;             /* number of failures in specific test */
-extern int  max_nmpt;		/* max. number of messages per test */
+#include "error.h"
+#include "tests.h"
 
 /*
  * Use for logging error conditions
