@@ -1,12 +1,12 @@
 /*********************************************************************
  *   Copyright 1996, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Id: test_write.c 11906 2005-08-08 19:51:43Z root $
+ *   $Id: test_write.c 15015 2006-11-03 17:39:01Z jimg $
  *********************************************************************/
 
 #include "tests.h"
-#include <math.h>
-
+#include "math.h"
+#include <stdio.h> /* For FILE stuff. */
 
 /*
  * Test nc_create
@@ -79,6 +79,7 @@ test_nc_create(void)
  *    put vars
  *    close
  *    check file: vars & atts
+ *    check reopening with NC_WRITE and adding new dims, atts, vars
  */
 void
 test_nc_redef(void)
@@ -90,10 +91,11 @@ test_nc_redef(void)
     int varid;         /* variable id */
     int varid1;        /* variable id */
     int err;
-    char * title = "Not funny";
+    const char * title = "Not funny";
     double var;
     char name[NC_MAX_NAME];
     size_t length;
+    int fmt_variant1, fmt_variant2;
 
 	/* BAD_ID tests */
     err = nc_redef(BAD_ID);
@@ -123,7 +125,7 @@ test_nc_redef(void)
         error("nc_create: %s", nc_strerror(err));
         return;
     }
-    /* limit for ncio implementations which which have infinite chunksize */
+    /* limit for ncio implementations which have infinite chunksize */
     if(sizehint > 32768)
 	sizehint = 16384;
     def_dims(ncid);
@@ -183,12 +185,15 @@ test_nc_redef(void)
     err = nc_put_var1_double(ncid, varid, NULL, &var);
     IF (err)
         error("nc_put_var1_double: %s", nc_strerror(err));
+    err = nc_inq_format(ncid, &fmt_variant1);
+    IF (err)
+	error("nc_inq_format: %s", nc_strerror(err));
     err = nc_close(ncid);
     IF (err) 
 	error("nc_close: %s", nc_strerror(err));
 
-	/* check scratch file written as expected */
-    check_file(scratch);
+    /* check scratch file written as expected */
+    check_file(scratch);  /* checks all except "abc" stuff added above */
     err = nc_open(scratch, NC_NOWRITE, &ncid);
     IF (err)
         error("nc_open: %s", nc_strerror(err));
@@ -208,6 +213,62 @@ test_nc_redef(void)
     IF (err)
         error("nc_close: %s", nc_strerror(err));
 
+    /* open scratch file for writing, add another dim, var, att, then check */
+    err = nc_open(scratch, NC_WRITE, &ncid);
+    IF (err)
+        error("nc_open: %s", nc_strerror(err));
+    err = nc_redef(ncid);
+    IF (err)
+        error("nc_redef: %s", nc_strerror(err));
+    err = nc_def_dim(ncid, "def", sizehint, &dimid);
+    IF (err)
+        error("nc_def_dim: %s", nc_strerror(err));
+    err = nc_def_var(ncid, "defScalar", NC_INT, 0, NULL, &varid);
+    IF (err)
+        error("nc_def_var: %s", nc_strerror(err));
+    err = nc_def_var(ncid, "def", NC_INT, 1, &dimid, &varid1);
+    IF (err)
+        error("nc_def_var: %s", nc_strerror(err));
+    err = nc_put_att_text(ncid, NC_GLOBAL, "Credits", 1+strlen("Thanks!"), 
+			  "Thanks!");
+    IF (err)
+        error("nc_put_att_text: %s", nc_strerror(err));
+    err = nc_enddef(ncid);
+    IF (err)
+        error("nc_enddef: %s", nc_strerror(err));
+    var = 2.0;
+    err = nc_put_var1_double(ncid, varid, NULL, &var);
+    IF (err)
+        error("nc_put_var1_double: %s", nc_strerror(err));
+    err = nc_close(ncid);
+    IF (err) 
+	error("nc_close: %s", nc_strerror(err));
+
+    check_file(scratch);
+    err = nc_open(scratch, NC_NOWRITE, &ncid);
+    IF (err)
+        error("nc_open: %s", nc_strerror(err));
+    err = nc_inq_dim(ncid, dimid, name, &length);
+    IF (err) 
+	error("nc_inq_dim: %s", nc_strerror(err));
+    IF (strcmp(name, "def") != 0) 
+	error("Unexpected dim name");
+    IF (length != sizehint) 
+	error("Unexpected dim length");
+    err = nc_get_var1_double(ncid, varid, NULL, &var);
+    IF (err)
+        error("nc_get_var1_double: %s", nc_strerror(err));
+    IF (var != 2.0)
+        error("nc_get_var1_double: unexpected value");
+    /* make sure format variant hasn't changed from when created */
+    err = nc_inq_format(ncid, &fmt_variant2);
+    IF (err)
+	error("nc_inq_format: %s", nc_strerror(err));
+    IF (fmt_variant1 != fmt_variant2)
+	error("nc_enddef changed format variant");
+    err = nc_close(ncid);
+    IF (err)
+        error("nc_close: %s", nc_strerror(err));
     err = remove(scratch);
     IF (err)
         error("remove of %s failed", scratch);
@@ -465,7 +526,7 @@ test_nc_def_dim(void)
 	IF (err != NC_EBADNAME)
 	    error("bad name: status = %d", err);
         err = nc_def_dim(ncid, dim_name[i], NC_UNLIMITED-1, &dimid);
-	IF (err != NC_EINVAL)
+	IF (err != NC_EDIMSIZE)
 	    error("bad size: status = %d", err);
         err = nc_def_dim(ncid, dim_name[i], dim_len[i], &dimid);
         IF (err) 
@@ -646,7 +707,6 @@ test_nc_def_var(void)
 }
 
 
-#ifdef TEST_VOIDSTAR
 /*
  * Test nc_put_var1
  */
@@ -902,7 +962,7 @@ test_nc_put_vars(void)
             if (var_dimid[i][j] > 0) {          /* skip record dim */
 		start[j] = var_shape[i][j];
 		err = nc_put_vars(ncid, i, start, edge, stride, buf);
-		IF (err != NC_EINVALCOORDS)
+		IF (err != NC_EINVALCOORDS && err != NC_EEDGE)
 		    error("bad index: status = %d", err);
 		start[j] = 0;
 		edge[j] = var_shape[i][j] + 1;
@@ -1076,7 +1136,7 @@ test_nc_put_varm(void)
             if (var_dimid[i][j] > 0) {          /* skip record dim */
 		start[j] = var_shape[i][j];
 		err = nc_put_varm(ncid, i, start, edge, stride, imap, buf);
-		IF (err != NC_EINVALCOORDS)
+		IF (err != NC_EINVALCOORDS && err != NC_EEDGE)
 		    error("bad index: status = %d", err);
 		start[j] = 0;
 		edge[j] = var_shape[i][j] + 1;
@@ -1154,7 +1214,6 @@ test_nc_put_varm(void)
     IF (err)
         error("remove of %s failed", scratch);
 }
-#endif /* TEST_VOIDSTAR */
 
 
 /*
@@ -1239,7 +1298,6 @@ test_nc_rename_var(void)
 }
 
 
-#ifdef TEST_VOIDSTAR
 void
 test_nc_put_att(void)
 {
@@ -1308,7 +1366,6 @@ test_nc_put_att(void)
     IF (err)
         error("remove of %s failed", scratch);
 }
-#endif /* TEST_VOIDSTAR */
 
 
 /*
@@ -1675,7 +1732,7 @@ test_nc_del_att(void)
 	    }
 	    err = nc_inq_varnatts(ncid, varid, &natts);
 	    IF (err)
-		error("nc_inq_natts: %s", nc_strerror(err));
+		error("nc_inq_varnatts: %s", nc_strerror(err));
 	    IF (natts != numatts-j-1)
 		error("natts: expected %d, got %d", numatts-j-1, natts);
 	}
@@ -1919,3 +1976,89 @@ test_nc_set_fill(void)
     IF (err)
         error("remove of %s failed", scratch);
 }
+
+/* This function gets the version of a netCDF file, 1 is for netCDF
+   classic, 2 for 64-bit offset format, (someday) 3 for HDF5 format.
+*/
+#define MAGIC_NUM_LEN 4
+static
+int
+nc_get_file_version(char *path, int *version)
+{
+   FILE *fp;
+   char magic[MAGIC_NUM_LEN];
+
+   /* Need two valid pointers - check for NULL. */
+   if (!version || !path)
+      return NC_EINVAL;
+
+   /* Figure out if this is a netcdf or hdf5 file. */
+   if (!(fp = fopen(path, "r")) ||
+       fread(magic, MAGIC_NUM_LEN, 1, fp) != 1)
+      return errno;
+   fclose(fp);
+   if (strncmp(magic, "CDF", MAGIC_NUM_LEN-1)==0)
+   {
+      if (magic[MAGIC_NUM_LEN-1] == NC_FORMAT_CLASSIC || 
+	  magic[MAGIC_NUM_LEN-1] == NC_FORMAT_64BIT)
+	 *version = magic[MAGIC_NUM_LEN-1];
+      else
+	 return NC_ENOTNC;
+   }
+   /*   tomorrow, tomorrow, I love you tomorrow, you're always a day
+	away! */
+   /*if (magic[1] == 'H' && magic[2] == 'D' && magic[3] == 'F')
+      *version = 3;*/
+   return NC_NOERR;
+}
+
+/*
+ * Test nc_set_default_format
+ *    try with bad default format
+ *    try with NULL old_formatp
+ *    try in data mode, check error
+ *    check that proper set to NC_FILL works for record & non-record variables
+ *    (note that it is not possible to test NC_NOFILL mode!)
+ *    close file & create again for test using attribute _FillValue
+ */
+void
+test_nc_set_default_format(void)
+{
+    int ncid;
+    int err;
+    int i;
+    int version;
+    int old_format;
+
+    /* bad format */
+    err = nc_set_default_format(BAD_DEFAULT_FORMAT, &old_format);
+    IF (err != NC_EINVAL)
+	error("bad default format: status = %d", err);
+
+    /* NULL old_formatp */
+    err = nc_set_default_format(NC_FORMAT_64BIT, NULL);
+    IF (err)
+	error("null old_fortmatp: status = %d", err);
+
+    /* Cycle through available formats. */
+    for(i=1; i<3; i++)
+    {
+       if ((err = nc_set_default_format(i, NULL)))
+	  error("setting classic format: status = %d", err);
+       if ((err=nc_create(scratch, NC_CLOBBER, &ncid)))
+	  error("bad nc_create: status = %d", err);
+       if ((err=nc_put_att_text(ncid, NC_GLOBAL, "testatt", 
+				sizeof("blah"), "blah")))
+	  error("bad put_att: status = %d", err);
+       if ((err=nc_close(ncid)))
+	  error("bad close: status = %d", err);
+       if ((err = nc_get_file_version(scratch, &version)))
+	  error("bad file version = %d", err);
+       if (version != i)
+	  error("bad file version = %d", err);
+    }
+
+    /* Remove the left-over file. */
+    if ((err = remove(scratch)))
+       error("remove of %s failed", scratch);
+}
