@@ -1,11 +1,11 @@
 /*********************************************************************
  *   Copyright 1996, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Id: util.c 11906 2005-08-08 19:51:43Z root $
+ *   $Id: util.c 15015 2006-11-03 17:39:01Z jimg $
  *********************************************************************/
 
-#include <math.h>
 #include "tests.h"
+#include <math.h>
 
 void
 print_nok(int nok)
@@ -44,7 +44,7 @@ inRange_uchar(const double value, const nc_type datatype)
     return inRange(value, datatype);
 }
 
-int
+static int
 inRange_float(const double value, const nc_type datatype)
 {
     double min, max;
@@ -339,8 +339,7 @@ hash( const nc_type type, const int rank, const size_t *index )
 		    case NC_BYTE:   return X_BYTE_MIN-1.0;
 		    case NC_SHORT:  return X_SHORT_MIN-1.0;
 		    case NC_INT:   return X_INT_MIN-1.0;
-		    case NC_FLOAT:  return X_FLOAT_MIN;
-	//   Reza-infinity       case NC_FLOAT:  return X_FLOAT_MIN * (1.0 + FUZZ); 
+		    case NC_FLOAT:  return X_FLOAT_MIN * (1.0 + FUZZ);
 		    case NC_DOUBLE: return -1.0;
 		    default:  assert(0);
 		}
@@ -350,8 +349,7 @@ hash( const nc_type type, const int rank, const size_t *index )
 		    case NC_BYTE:   return X_BYTE_MAX+1.0;
 		    case NC_SHORT:  return X_SHORT_MAX+1.0;
 		    case NC_INT:   return X_INT_MAX+1.0;
-		    case NC_FLOAT:  return X_FLOAT_MAX;
-	//   Reza-infDAS         case NC_FLOAT:  return X_FLOAT_MAX * (1.0 + FUZZ);
+		    case NC_FLOAT:  return X_FLOAT_MAX * (1.0 + FUZZ);
 		    case NC_DOUBLE: return 1.0;
 		    default:  assert(0);
 		}
@@ -626,12 +624,12 @@ put_vars(int ncid)
 	if (var_name[i][0] == 'c') {
 	    err = nc_put_vara_text(ncid, i, start, var_shape[i], text);
 	    IF (err)
-		error("nc_put_att_text: %s", nc_strerror(err));
+		error("nc_put_vara_text: %s", nc_strerror(err));
 	} else {
 	    err = nc_put_vara_double(ncid, i, start, var_shape[i], value);
 	    if (allInRange) {
 		IF (err)
-		    error("nc_put_att_text: %s", nc_strerror(err));
+		    error("nc_put_vara_double: %s", nc_strerror(err));
 	    } else {
 		IF (err != NC_ERANGE)
 		    error("type-conversion range error: status = %d", err);
@@ -659,6 +657,7 @@ write_file(char *filename)
     IF (err) 
 	error("nc_enddef: %s", nc_strerror(err));
     put_vars(ncid);
+
     err = nc_close (ncid);
     IF (err) 
 	error("nc_close: %s", nc_strerror(err));
